#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>  
#include <strings.h>
#include <malloc.h>

#include "analyse_alti.h"
#include "aktarus.h"

#include "gsl/gsl_cblas.h"
#include "gsl/gsl_linalg.h"
#include "gsl/gsl_vector.h"
#include "gsl/gsl_matrix.h"
#include "gsl/gsl_permutation.h"

#include <pthread.h>  /*parra*/
//#include <thread.h>

#define GSL 1

/*constantes globales de Tidefunctions.c*/
double omega_T=  13149000.0;
double omega_s=    481267.892;
double omega_h=     36000.76892; 
double omega_p=      4069.0322056;
double omega_n=      1934.1423972;
double omega_p1=        1.719175;

/*liste des functions 
			nodal_factort
			astronomic_angle
			init_argument
			tides julianday
			tides_const_init
			tide_wavearray_init
			pulsation
			greenwhich_argument
			harmonic_analysis
			Tide_prediction
                        orthotide_analysis 
			
			calendary
			getcnesdate
			*/

/*XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX*/

double nodal_factort(int formula, astro_ang_struct *astro_ang)

/*XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX*/
{
  double s,f;

  switch (formula)
    {

/* formule 0, solar waves */
  
    case 0:
      f=1.0;
      break;
   
/* formule 1, compound waves (78 x 78)*/
  
    case 1:
      f=nodal_factort(78,astro_ang)*nodal_factort(78,astro_ang);
      break;

/* formule 2, compound waves (78 x 0)  ===  (78)*/
  
    case 2:
      f=nodal_factort(78,astro_ang)*nodal_factort(0,astro_ang);
      break;
  
  
  
/* formule 4,  compound waves (78 x 235) */
  
    case 4:
      f=nodal_factort(78,astro_ang)*nodal_factort(235,astro_ang);
      break;

  
/* formule 5,  compound waves (78 *78 x 235) */
  
    case 5:
      f=nodal_factort(78,astro_ang)*nodal_factort(78,astro_ang)*nodal_factort(235,astro_ang);
      break;
  
/* formule 6,  compound waves (78 *78 x 0) */
  
    case 6:
      f=nodal_factort(78,astro_ang)*nodal_factort(78,astro_ang)*nodal_factort(0,astro_ang);
      break;
  
/* formule 7, compound waves (75 x 75)*/
  
    case 7:
      f=nodal_factort(75,astro_ang)*nodal_factort(75,astro_ang);
      break;
      
/* formule 8,  compound waves (78 x 0 x 235) */
  
    case 8:
      f=nodal_factort(78,astro_ang)*nodal_factort(0,astro_ang)*nodal_factort(235,astro_ang);
      break;
  
/* formule 9,  compound waves (78 x 0 x 227) */
  
    case 9:
      f=nodal_factort(78,astro_ang)*nodal_factort(0,astro_ang)*nodal_factort(227,astro_ang);
      break;
  
/* formule 10,  compound waves (78 x 227) */
  
    case 10:
      f=nodal_factort(78,astro_ang)*nodal_factort(227,astro_ang);
      break;
  
/* formule 11,  compound waves (75 x 0) */
  
    case 11:
      f=nodal_factort(75,astro_ang)*nodal_factort(0,astro_ang);
      break;
      
/* formule 12,  compound waves (78 x 78 x 78 x 0) */
  
    case 12:
      f=nodal_factort(78,astro_ang)*nodal_factort(78,astro_ang)*nodal_factort(78,astro_ang)*nodal_factort(0,astro_ang);
      break;
      
/* formule 13, compound waves (78 x 75)*/
  
    case 13:
      f=nodal_factort(78,astro_ang)*nodal_factort(75,astro_ang);
      break;
  
/* formule 14, compound waves (235 x 0)  ===  (235)*/
  
    case 14:
      f=nodal_factort(235,astro_ang)*nodal_factort(0,astro_ang);
      break;
  
/* formule 15, compound waves (235 x 75) */
  
    case 15:
      f=nodal_factort(235,astro_ang)*nodal_factort(75,astro_ang);
      break;
  
/* formule 16, compound waves (78 x 0 x 0)  ===  (78)*/
  
    case 16:
      f=nodal_factort(78,astro_ang)*nodal_factort(0,astro_ang)*nodal_factort(0,astro_ang);
      break;
      
/* formule 17,  compound waves (227 x 0) */
  
    case 17:
      f=nodal_factort(227,astro_ang)*nodal_factort(0,astro_ang);
      break;
      
/* formule 18,  compound waves (78 x 78 x 78 ) */
  
    case 18:
      f=nodal_factort(78,astro_ang)*nodal_factort(78,astro_ang)*nodal_factort(78,astro_ang);
      break;
  
/* formule 19, compound waves (78 x 0 x 0 x 0)  ===  (78)*/
  
    case 19:
      f=nodal_factort(78,astro_ang)*nodal_factort(0,astro_ang)*nodal_factort(0,astro_ang);
      break;
      

/* formule 73 */
  
    case 73:
      s=sin(astro_ang->sh_I);
      f=(2./3.-s*s)/0.5021;
      break;

/* formule 74 */
  
    case 74:
      s=sin(astro_ang->sh_I);
      f=s*s/0.1578;
      break;
  
/* formule 75 */
  
    case 75:
      s=cos (astro_ang->sh_I/2);
      f=sin (astro_ang->sh_I)*s*s/0.3800;
      break;  

/* formule 76 */
  
    case 76:
      f=sin (2*astro_ang->sh_I)/0.7214;
      break;
  
/* formule 77 */
  
    case 77:
      s=sin (astro_ang->sh_I/2);
      f=sin (astro_ang->sh_I)*s*s/0.0164;
      break;
  
/* formule 78 */
  
    case 78:
      s=cos (astro_ang->sh_I/2);
      f=s*s*s*s/0.9154;
      break;

/* formule 79 */
    
    case 79:
      s=sin(astro_ang->sh_I);
      f=s*s/0.1565;
      break;
  
/* formule 144 */
  
    case 144:
      s=sin (astro_ang->sh_I/2);
      f=(1-10*s*s+15*s*s*s*s)*cos(astro_ang->sh_I/2)/0.5873;
      break;

/* formule 149 */
  
    case 149:
      s=cos (astro_ang->sh_I/2);
      f=s*s*s*s*s*s/0.8758;
      break;

/* formule 215 */
  
    case 215:
      s=cos (astro_ang->sh_I/2);
      f=s*s*s*s/0.9154*astro_ang->sh_x1ra;
      break;
  
/* formule 227 */
  
    case 227:
      s=sin (2*astro_ang->sh_I);
      f=sqrt (0.8965*s*s+0.6001*s*cos (astro_ang->sh_nu)+0.1006);
      break;

/* formule 235 */
   
    case 235:
      s=sin (astro_ang->sh_I);
      f=sqrt (19.0444*s*s*s*s+2.7702*s*s*cos (2*astro_ang->sh_nu)+.0981);
      break;

 
    default:
      exit;
    }
  return(f);
}


/*XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX*/

  void astronomic_angle(double tj, int verbose, astro_ang_struct *astro_ang)

/*XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX*/


/*----------------------------------------------------------------------

  tj is time elapsed since 1st January 1900, 0 hour, counted in julian
  century (e.g. time in days divide by 36525)
----------------------------------------------------------------------*/

{
  double dtr,days,tt;
  date_t actual;
  double cosI,p,q;
  double t2,t4,sin2I,s2;
  double tgI2,P;
  double sh_tgn2,at1,at2;

  pi=acos(-1.0);
  dtr=pi/180.0;

/*----------------------------------------------------------------------
  Sh_n Longitude of ascending lunar node
----------------------------------------------------------------------*/

  astro_ang->sh_N=(259.1560563 -1934.1423972 *tj)*dtr;

/*----------------------------------------------------------------------
 T mean solar angle (Greenwhich time)
----------------------------------------------------------------------*/
  days=36525.0*tj;
  astro_ang->sh_T=((days - (int)days)*24.0*15.0+180.0)*dtr;

/*----------------------------------------------------------------------
 h mean solar Longitude
----------------------------------------------------------------------*/

  astro_ang->sh_h=(280.1895015 +36000.76892 *tj)*dtr;

/*----------------------------------------------------------------------
 s mean lunar Longitude
----------------------------------------------------------------------*/

  astro_ang->sh_s=(277.0256206 +481267.892 *tj)*dtr;

/*----------------------------------------------------------------------
 p1 Longitude of solar perigee
----------------------------------------------------------------------*/

  astro_ang->sh_p1=(281.2208568 +1.719175 *tj)*dtr;

/*----------------------------------------------------------------------
 p Longitude of lunar perigee
----------------------------------------------------------------------*/

  astro_ang->sh_p=(334.3837214 +4069.0322056 *tj)*dtr;

  astro_ang->sh_N =fmod(astro_ang->sh_N ,2*pi);
  astro_ang->sh_s =fmod(astro_ang->sh_s ,2*pi);
  astro_ang->sh_h =fmod(astro_ang->sh_h, 2*pi);
  astro_ang->sh_p =fmod(astro_ang->sh_p, 2*pi);
  astro_ang->sh_p1=fmod(astro_ang->sh_p1,2*pi);

  cosI=0.913694997 -0.035692561 *cos(astro_ang->sh_N);

  astro_ang->sh_I=acos(cosI);

  sin2I=sin(astro_ang->sh_I);
  sh_tgn2=tan(astro_ang->sh_N/2.0);
  
  at1=atan(1.01883*sh_tgn2);
  at2=atan(0.64412*sh_tgn2);
  
  astro_ang->sh_xi=-at1-at2+astro_ang->sh_N;

  if (astro_ang->sh_N > pi) astro_ang->sh_xi=astro_ang->sh_xi-2.0*pi;

  astro_ang->sh_nu=at1-at2;

/*----------------------------------------------------------------------
 For constituents l2 k1 k2
----------------------------------------------------------------------*/

  tgI2=tan(astro_ang->sh_I/2.0);
  P=astro_ang->sh_p-astro_ang->sh_xi;
  
  t2=tgI2*tgI2;
  t4=t2*t2;
  astro_ang->sh_x1ra=sqrt(1.0-12.0*t2*cos(2.0*P)+36.0*t4);
  
  p=sin(2.0*P);
  q=1.0/(6.0*t2)-cos(2.0*P);
  astro_ang->sh_R=atan(p/q);
  
  p=sin(2.0*astro_ang->sh_I)*sin(astro_ang->sh_nu);
  q=sin(2.0*astro_ang->sh_I)*cos(astro_ang->sh_nu)+0.3347;
  astro_ang->sh_nuprim=atan(p/q);
  
  s2=sin(astro_ang->sh_I)*sin(astro_ang->sh_I);
  p=s2*sin(2.0*astro_ang->sh_nu);
  q=s2*cos(2.0*astro_ang->sh_nu)+0.0727;
  astro_ang->sh_nusec=0.5*atan(p/q); 

  tt=(tj*36525.-18262)*24;
  getcnesdate(tt,&actual);


  if (verbose)
    {
    printf ("%d/%d/%d \n",actual.day,actual.month,actual.year);
    printf ("s: %f h: %f p: %f p1: %f \n",astro_ang->sh_s/dtr,astro_ang->sh_h/dtr,astro_ang->sh_p/dtr,astro_ang->sh_p1/dtr);
    printf ("I: %f N: %f \n",astro_ang->sh_I/dtr,astro_ang->sh_N/dtr);
    }
 
}

			
			
/*XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX*/

  void init_argument(double first_time, int verbose, int CTO,astro_ang_struct *astro_ang)

/*XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX*/

{
  int i,N;

  double scale=36525*24.0;

  date_t t_schureman={1,1,1900,0.0};    /* LR: 21/07/2003, -43200 -> 0 */
  date_t date58=     { 1, 1, 1958, 0.0};/* LR: 21/07/2003, -43200 -> 0 */ 
  date_t date50=     { 1, 1, 1950, 0.0};/* LR: 21/07/2003, -43200 -> 0 */
  date_t date85=     { 1, 1, 1985, 0.0};/* LR: 21/07/2003, -43200 -> 0 */
  date_t date2000=     { 1, 1, 2000, 0.0};/* LR: 21/07/2003, -43200 -> 0 */
  date_t date;
  double tj;

  
  switch (CTO)
    {
    case(0): {date =date50;break;}
    case(1): {date =date58;break;}
    case(2): {date =date85;break;}
    case(3): {date =date2000;break;}
    default: {printf("error in time reference, exit\n");exit(-1);}
    }
  
  N=  julian_day(date.month,date.day,date.year)
     -julian_day(t_schureman.month,t_schureman.day,t_schureman.year);
  /* number of day elapsed between 1950 and 1900 (CNES Time and SCHUREMAN Time) */
  /*or number of day elapsed between 1958 and 1900 (CTO Time and SCHUREMAN Time) */
  /*or number of day elapsed between 1985 and 1900 (ESA Time and SCHUREMAN Time) */

  tj=((double) N + first_time/(double) 24.) /(double) 36525.0;

  astronomic_angle(tj,verbose,astro_ang);

}
		      

/*XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX*/

int julian_day(int mm,int id,int iyyy)

/*XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX*/

/*-----------------------------------------------------------------------
  tides_julianday routine returns the julian day number which begins at noon
  of the calendar date specified by month mm, day id, and year iyyy.
  positive year signifies a.d.; negative, b.c.  (remember that the
  year after 1 b.c. was 1 a.d.
  this routine has been lifted directly from the book
  press et al., numerical recipes, cambridge univ. press, 1986.
  routine handles changeover to gregorian calendar on oct. 15, 1582.
  note: to get the corresponding modified julian day,
        set mjd = julday - 2400001.

	f77 to C: 7/10/2001 (Thierry)
-----------------------------------------------------------------------*/

  {
   int igreg=15+31*(10+12*1582);
   int jy,jm,ja, tmp_iyyy;
   double temp_tides_juliandays;
   
   tmp_iyyy=iyyy;
   
   if (tmp_iyyy != 0)
/*      if (tmp_iyyy == 2000) printf("Probleme avec le passage a l annee 2000"); */
/*      else  */
       { 
       if (tmp_iyyy < 0) tmp_iyyy = tmp_iyyy + 1;
       if (mm > 2)
         {
	 jy = tmp_iyyy;
	 jm = mm + 1;
	 }
       else
         {
	 jy = tmp_iyyy - 1;
	 jm = mm + 13;
	 }
       temp_tides_juliandays=floor(365.25*jy)+floor(30.6001*jm)+id+1720995;
       if(id+31*(mm+12*tmp_iyyy) >= igreg)
         {
	 ja=floor(0.01*jy);
         temp_tides_juliandays=temp_tides_juliandays+2-ja+floor(0.25*ja);
	 }
       }/*else*/
       
    return(temp_tides_juliandays);   
    
  }/*end*/
  
  

/*XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX*/

void tide_wavearray_initC(wavelist *spectrum_list, int nwave)

/*XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX*/
  {
    
    int code,i,tmp,addnul=0;
    char a,filename[256] ;
    FILE  *spectrefile;
    double scale=36525*24.0;
    
    sprintf(filename,"ondes%d.dut",nwave);
    spectrefile=fopen(filename,"r");
    if(spectrefile==NULL) {printf("wave file not found");exit(-1);};
    fscanf(spectrefile,"%d ",&tmp);
    spectrum_list->n=tmp;
    
    for(i=0;i<spectrum_list->n;i++)
      {
	fscanf(spectrefile,"%d ",&code);

	if(code==999)
	  {
	    spectrum_list->wave[i]=wNUL;
	    spectrum_list->wave[i].nT=addnul;addnul++;
	    fscanf(spectrefile,"%s %c %lf ",&spectrum_list->wave[i].name,&a,&spectrum_list->wave[i].omega);
	  }
	do {a=fgetc(spectrefile);} while ( a!= '\n');
 
/*-------------------------- Longues periodes ---------------------------*/

   if(code == 47) (spectrum_list->wave[i]=wSa);

   else if(code == 46)(spectrum_list->wave[i]=wSsa);

   else if(code == 41)(spectrum_list->wave[i]=wMSm);

   else if(code == 38)(spectrum_list->wave[i]=wMm);

   else if(code == 39)(spectrum_list->wave[i]=wMSf);

   else if(code == 40)(spectrum_list->wave[i]=wMf);

   else if(code == 44)(spectrum_list->wave[i]=wMStm);

   else if(code == 42)(spectrum_list->wave[i]=wMtm);

   else if(code == 43)(spectrum_list->wave[i]=wMqm);
   
   else if(code == 45)(spectrum_list->wave[i]=wMSqm);

/*------------------------------- Diurnes -------------------------------*/

   else if(code == 67)(spectrum_list->wave[i]=w2Q1);

   else if(code == 68)(spectrum_list->wave[i]=wSig1);

   else if(code == 27)(spectrum_list->wave[i]=wQ1);

   else if(code == 69)(spectrum_list->wave[i]=wRo1);

   else if(code == 1)(spectrum_list->wave[i]=wO1);

 /*  else if(code == 121)(spectrum_list->wave[i]=wTau1);*/

   else if(code == 74)(spectrum_list->wave[i]=wM1);

   else if(code == 70)(spectrum_list->wave[i]=wKi1);

   else if(code == 71)(spectrum_list->wave[i]=wPi1);

   else if(code == 2)(spectrum_list->wave[i]=wP1);

   else if(code == 3)(spectrum_list->wave[i]=wK1);

   else if(code == 0)(spectrum_list->wave[i]=wPsi1);

   else if(code == 72)(spectrum_list->wave[i]=wPhi1);

   else if(code == 26)(spectrum_list->wave[i]=wS1);

   else if(code == 73)(spectrum_list->wave[i]=wTta1);

   else if(code == 29)(spectrum_list->wave[i]=wJ1);

   else if(code == 28)(spectrum_list->wave[i]=wOO1);

  /* else if(code == 170)(spectrum_list->wave[i]=wNu1);*/

   /* else if(code == 75)(spectrum_list->wave[i]=wM12);*/

   else if(code == 49)(spectrum_list->wave[i]=wMP1);

   else if(code == 48)(spectrum_list->wave[i]=wSO1);

   else if(code == 76)(spectrum_list->wave[i]=wKQ1);

/*---------------------------- Semi-diurnes -----------------------------*/

   /*else if(code == 200)(spectrum_list->wave[i]=wEps2);*/

   else if(code == 4)(spectrum_list->wave[i]=wE2);

   else if(code == 5)(spectrum_list->wave[i]=w2N2);

   else if(code == 6)(spectrum_list->wave[i]=wMu2);

   else if(code == 7)(spectrum_list->wave[i]=wN2);

   else if(code == 8)(spectrum_list->wave[i]=wNu2);

   /*else if(code == 230)(spectrum_list->wave[i]=wGamma2);*/

   /*else if(code == 231)(spectrum_list->wave[i]=wA2);*/

   else if(code == 9)(spectrum_list->wave[i]=wM2);

   /*else if(code == 233)(spectrum_list->wave[i]=wDelta2);*/

   else if(code == 10)(spectrum_list->wave[i]=wLa2);

   else if(code == 11)(spectrum_list->wave[i]=wL2);

   else if(code == 12)(spectrum_list->wave[i]=wT2);

   else if(code == 13)(spectrum_list->wave[i]=wS2);

   else if(code == 50)(spectrum_list->wave[i]=wR2);

   else if(code == 14)(spectrum_list->wave[i]=wK2);

   else if(code == 77)(spectrum_list->wave[i]=wKJ2);

   /*else if(code == 260)(spectrum_list->wave[i]=wEta2);*/

   else if(code == 51)(spectrum_list->wave[i]=wOQ2);

   else if(code == 65)(spectrum_list->wave[i]=w2MK2);

   else if(code == 31)(spectrum_list->wave[i]=wMSK2);

   else if(code == 15)(spectrum_list->wave[i]=wMSN2);

   else if(code == 16)(spectrum_list->wave[i]=w2SM2);

   else if(code == 37)(spectrum_list->wave[i]=wM_SK_2);

   else if(code == 36)(spectrum_list->wave[i]=wM_KS_2);

   else if(code == 30)(spectrum_list->wave[i]=wMKS2);

   else if(code == 100)(spectrum_list->wave[i]=wOP2);

   else if(code == 101)(spectrum_list->wave[i]=wMNS2);

/*---------------------------- Tiers-diurnes -----------------------------*/

   else if(code == 34)(spectrum_list->wave[i]=wM3);

   else if(code == 35)(spectrum_list->wave[i]=wS3);

   else if(code == 24)(spectrum_list->wave[i]=wMK3);

   else if(code == 25)(spectrum_list->wave[i]=w2MK3);

   else if(code == 53)(spectrum_list->wave[i]=wSO3);

   else if(code == 54)(spectrum_list->wave[i]=wSK3);

   else if(code == 102)(spectrum_list->wave[i]=wMO3);

/*---------------------------- Quart-diurnes -----------------------------*/

   else if(code == 33)(spectrum_list->wave[i]=wN4);

   else if(code == 17)(spectrum_list->wave[i]=wMN4);

   else if(code == 18)(spectrum_list->wave[i]=wM4);

   else if(code == 19)(spectrum_list->wave[i]=wMS4);

   else if(code == 20)(spectrum_list->wave[i]=wMK4);

   else if(code == 56)(spectrum_list->wave[i]=wS4);

   else if(code == 55)(spectrum_list->wave[i]=wSN4);

   else if(code == 58)(spectrum_list->wave[i]=w3MS4);

   else if(code == 103)(spectrum_list->wave[i]=wSK4);

   /*else if(code == 500)(spectrum_list->wave[i]=wM5);*/

/*---------------------------- Sixime-diurnes -----------------------------*/

   else if(code == 21)(spectrum_list->wave[i]=w2MN6);

   else if(code == 22)(spectrum_list->wave[i]=wM6);

   else if(code == 59)(spectrum_list->wave[i]=w2MS6);

   else if(code == 60)(spectrum_list->wave[i]=w2MK6);

   else if(code == 23)(spectrum_list->wave[i]=wMSN6);
   
   else if(code == 104)(spectrum_list->wave[i]=w2SM6);

   else if(code == 105)(spectrum_list->wave[i]=wMSK6);
   
   /*---------------------------- Huitime-diurnes -----------------------------*/

   else if(code == 61)(spectrum_list->wave[i]=w3MS8);


   else if(code == 187)(spectrum_list->wave[i]=wmean);

  }/*for*/
  
  for(i=0;i<spectrum_list->n;i++)
    {
    if(spectrum_list->wave[i].omega == 0.)
      {
      spectrum_list->wave[i].omega = omega_T  *spectrum_list->wave[i].nT
                           + omega_s  *spectrum_list->wave[i].ns
                           + omega_h  *spectrum_list->wave[i].nh
                           + omega_p  *spectrum_list->wave[i].np
                           + omega_p1 *spectrum_list->wave[i].np1;
      spectrum_list->wave[i].omega /= scale;
      }
    printf ("wave: %10s, pulsation: %12.11f degrees/h \n", 
                                     spectrum_list->wave[i].name,spectrum_list->wave[i].omega);
    }

    }/*end*/
 
/*XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX*/
   
double pulsation( tidal_wave wave)

/*XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX*/
/* redundant with tide_wavearray_initC */

{
  int verbose,dummy=0;
  double scale=36525*24.0;
  double omega;

  omega = omega_T * wave.nT
        + omega_s * wave.ns
        + omega_h * wave.nh
        + omega_p * wave.np
        + omega_p1* wave.np1;
  omega /= scale;
  return(omega);
}

    
/*XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX*/

  double greenwhich_argument(tidal_wave w,astro_ang_struct *astro_ang)

/*XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX*/
/*----------------------------------------------------------------------

  Returns the phase of the tidal potential relative to the Greenwhich 
  meridian (e.g. the position of the fictuous celestial body). Units are
  radian.

----------------------------------------------------------------------*/

{
  double V0;

  V0=astro_ang->sh_T*w.nT+astro_ang->sh_s*w.ns+astro_ang->sh_h*w.nh+astro_ang->sh_p*w.np+astro_ang->sh_p1*w.np1+w.shift*deg_to_rad;
  V0=fmod(V0,pi2);
  return(V0);

}


/*XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX*/

  double nodal_phase(tidal_wave w,astro_ang_struct *astro_ang)

/*XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX*/
/*----------------------------------------------------------------------

  Returns the phase correction u due to nodal motion. Units are radian.

----------------------------------------------------------------------*/

{
  double u;

  u=astro_ang->sh_xi*w.nksi+astro_ang->sh_nu*w.nnu0+astro_ang->sh_nuprim*w.nnu1+astro_ang->sh_nusec*w.nnu2+astro_ang->sh_R*w.R;
  return(u);

}



/*XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX*/

  void harmonic_analysis(double *sealevel,double *time, int n,wavelist s,int nopt, double lon, double lat, fcomplex *tabZ_sortie, double *h, int *KEEP,int CTO)

/*XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX*/
/* ---------------------------------------------------------------------------
  
   Makes an harmonic analysis from a classical harmonic formalism;
   returns amplitude a and phase lag G from a sealevel time serie and for a specified wave list;
   !! Need a KEEP array specifying wave(s) that will be deduced by admittance 

-------------------------------------------------------------------------------*/
{
  int i,k,l,m,neq,status,job,step,nw,cnt;
  int *pivot;
  int info;
  int ldl,ldr;
  int stepmax=300; 
  int verbose=0; 
  int code,ci,toto;
     
  gsl_matrix *M;
  gsl_permutation *P;
  gsl_vector *B,*X;
  int signum; 

  double *A,*b;
  double **cs,**sn;
  double V,V0,dph2rps,omega,f,niveau0=0;
  double *zr,*zi,mean;
  double  *svals;
  double  *e, *lsvec, *rsvec;
  double *predic,crit,spmoy,predicmoy;
  double kappa,Ap1;

  FILE *matrice;
  
  char carac[256];

  tidal_wave wave;
  astro_ang_struct astro_ang;
   
  neq=2*s.n;



   /* allocation */
/*-----------------------------------------------------*/
  predic=(double *) calloc(n,sizeof(double)); /*modif le 10/06/2004 Thierry de malloc a calloc*/

  	
  M=gsl_matrix_calloc(neq,neq);
  X=gsl_vector_calloc(neq);
  B=gsl_vector_calloc(neq);
  P=gsl_permutation_calloc(neq);


  A= (double *)malloc(neq*neq*sizeof(double));
  b= (double *)malloc(neq*sizeof(double));
  zr=(double *) malloc(s.n*sizeof(double));
  zi=(double *) malloc(s.n*sizeof(double));

  cs=(double **) malloc(s.n*sizeof(double));
  sn=(double **) malloc(s.n*sizeof(double));
  for(k=0; k < s.n; k++) cs[k]=(double *) malloc(n*sizeof(double));
  for(k=0; k < s.n; k++) sn[k]=(double *) malloc(n*sizeof(double));
  pivot=(int *)malloc(neq*sizeof(int));
  e=(double *)malloc(neq*sizeof(double));
/*-----------------------------------------------------*/

  /* mean level retiring */
/*-----------------------------------------------------*/
  mean=0.;
  for(m=0; m<n; m++)
    {
    h[m]=sealevel[m];
    mean+=h[m];
    }
  mean/=n;
  for(m=0; m<n; m++)
    {
    h[m]-=mean;
    }
/*-----------------------------------------------------*/

  /* initialisation */
/*-----------------------------------------------------*/

  dph2rps=pi/(180.0*3600.0);

  nw=s.n; 
  for(k=0; k<neq; k++)for(l=0; l<neq; l++) A[k*neq+l]=0;

  for(k=0; k<nw; k++) zr[k]=0;
  for(k=0; k<nw; k++) zi[k]=0;
  for(k=0; k<s.n; k++) b[k]=0;
/*-----------------------------------------------------*/




  /* sin and cos building */
/*-----------------------------------------------------*/
  for(k=0; k < s.n; k++)
    {
    wave=s.wave[k]; 
    for(m=0; m<n; m++)
      {
	init_argument(time[m],verbose,CTO,&astro_ang);
     	f=nodal_factort(wave.formula,&astro_ang);
	V=greenwhich_argument(wave,&astro_ang)+nodal_phase(wave,&astro_ang);
/* 	V=fmod(wave.omega*time[m]*pi2/360,pi2); */
       cs[k][m]=f*cos(V);
       sn[k][m]=f*sin(V);
      }
    } 
/*-----------------------------------------------------*/


/*-----------------------------------------------------*/
  /* matrix A building */
/*-----------------------------------------------------*/
  for(k=0; k < s.n; k++)
    {
    for(l=0; l<s.n; l++) 
      {
      for(m=0; m<n; m++)   
         {
	   A[2*l*neq    +2*k]  +=cs[k][m]*cs[l][m];
	   A[2*l*neq    +2*k+1]+=sn[k][m]*cs[l][m];
	   A[(2*l+1)*neq+2*k]  +=cs[k][m]*sn[l][m];
	   A[(2*l+1)*neq+2*k+1]+=sn[k][m]*sn[l][m];
	 }
      }
    }
/*-----------------------------------------------------*/


  /* admittance processing */
/*-----------------------------------------------------*/
 for(k=0; k < s.n; k++)
     if (KEEP[k]==1)
       {  
	 for(l=0; l<s.n; l++) 
	   {
	     A[2*l*neq    +2*k]  =0;
	     A[2*l*neq    +2*k+1]=0;/*toutes la ligne a 0*/
	     A[(2*l+1)*neq+2*k]  =0;
	     A[(2*l+1)*neq+2*k+1]=0;
	   }
	 A[2*k*neq+2*k]  =-1;      /*la diagonale a -1*/
	 A[(2*k+1)*neq+2*k+1]  =-1;
	 
	 code=s.wave[k].admit;
	 ci=0;     
	 while(code!=s.wave[ci].code){ci++;/*if(ci>=s.n)break;*/}
	 Ap1=s.wave[ci].Ap;
	 kappa=s.wave[k].Ap/Ap1;
	 A[2*ci*neq+2*k]  =kappa;  /*rapport des potential*/
	 A[(2*ci+1)*neq+2*k+1]  =kappa;
       }/*if keep*/
/*-----------------------------------------------------*/

/*-----------------------------------------------------*/
  /* matrix M building */
/*-----------------------------------------------------*/
  for(k=0; k <neq; k++) for(l=0; l<neq;l++)  gsl_matrix_set(M,l,k,A[k*neq+l]);
 
/*-----------------------------------------------------*/




  /* affichage de la matrice*/
/*-----------------------------------------------------*/
/*  sprintf(carac,"./mharmo.dat.%d",nopt); */
/*   matrice=fopen(carac,"w"); */
/*   fprintf(matrice,"coordonnes: %7.4f  %7.4f \n",lon, lat ); */
/*  fprintf(matrice,"\n"); */
/*   fprintf(matrice,"    "); */
/*   for(k=0;k<2*s.n;k++)  */
/*     {       */
/*       cnt=floor(k/2); */
/*       fprintf(matrice,"%5s",s.wave[cnt].name); */
/*     } */
/*   fprintf(matrice,"\n"); */
/*   for(k=0;k<s.n*2;k++) */
/*     { */
/*       cnt=floor(k/2); */
/*       fprintf(matrice,"%3s ",s.wave[cnt].name); */
/*       for(i=0;i<4*s.n*s.n;i+=2*s.n) */
/* 	{ */
/* 	  fprintf(matrice,"%4.0f ",A[i+k]); */
/* 	} */
/*       fprintf(matrice,"\n"); */
/*       fprintf(matrice,"----------------------------------------------------------------------"); */
/*       fprintf(matrice,"----------------------------------------------------------------------"); */
/*       fprintf(matrice,"----------------------------------\n"); */
/*     } */
/*   fclose(matrice); */
/*-----------------------------------------------------*/

  /* affichage des vecteurs propres */
/*-----------------------------------------------------*/

 /*  lsvec=malloc(neq*neq*sizeof(double)); */
/*   rsvec=malloc(neq*neq*sizeof(double)); */

/*   status=1; */
/*   dsvdc(A, neq, neq, neq, svals, e, lsvec, ldl=neq, rsvec,ldr=neq, job=11, &info); */


/*   sprintf(carac,"valp.dat.%d",nopt); */
/*   matrice=fopen(carac,"w"); */
/*   for(i=0;i<neq;i++)  fprintf(matrice," %lf  \n",svals[i]); */
/*   fclose(matrice); */
/*-----------------------------------------------------*/

  /* matrix A factorization */
/*-----------------------------------------------------*/
#ifdef LINPACKF_
  dchdc_(A,&neq,&neq, work,pivot,&status, &info);
#elif ESSL
  dgef  (A,&neq,&neq,pivot,&status);
#elif LINPACKF
  dgefa (A,&neq,&neq,pivot,&status);
#elif LINPACKC
  dgefa (A,neq,neq,pivot,&status);
#elif GSL
  toto=gsl_linalg_LU_decomp(M,P,&signum);
#endif
/*-----------------------------------------------------*/


/*--------------------------------*/
/*   analyse iterative            */
/*--------------------------------*/
/*  gsl_matrix_transpose(M);*/
crit=1;
step=0;
while((crit > 1.e-25)&&(step <= stepmax))
  {
    for(k=0; k<neq; k++) b[k]=0;
    for(k=0; k < s.n; k++)
      {
	for(m=0; m<n; m++)
	  {
	    b[2*k]  +=cs[k][m]*h[m];      /*construction second membre*/
	    b[2*k+1]+=sn[k][m]*h[m];
	  }
      }
for(k=0; k < s.n; k++)
     if (KEEP[k]==1)
       {  
	    b[2*k]  =0;      /*construction second membre admitance*/
	    b[2*k+1]=0;
       }
/*-----------------------------------------------------*/

/*--------------------------------*/
/*construction de B second membre */
/*--------------------------------*/

for(k=0; k <neq; k++) gsl_vector_set(B,k,b[k]);
/*-----------------------------------------------------*/




  /* resolution Ax=b */
/*-----------------------------------------------------*/
    job=0;
#ifdef LINPACKF_
    dgesl_ (A,&neq,&neq,pivot,b,&job);
#elif ESSL
    dges   (A,&neq,&neq,pivot,b);
#elif LINPACKF
    dgesl (A,&neq,&neq,pivot,b,&job);
#elif LINPACKC
   dgesl  (A,neq,neq,pivot,b,job);
#elif GSL
   toto=gsl_linalg_LU_solve(M,P,B,X);
#endif
/*-----------------------------------------------------*/


for(k=0; k <neq; k++) b[k]=gsl_vector_get(X,k);


   for(k=0; k < s.n; k++)                /* iteration */
     {
       zr[k]+=b[2*k];
       zi[k]+=b[2*k+1];
     }
   for(m=0; m<n; m++)
     {
       for(k=0; k < s.n; k++)
	 {
	   h[m]-=cs[k][m]*b[2*k]+sn[k][m]*b[2*k+1];      /* residu*/
/* 	   predic[m]=cs[k][m]*b[2*k]+sn[k][m]*b[2*k+1];   */
	   predic[m]+=cs[k][m]*b[2*k]+sn[k][m]*b[2*k+1];  /*modif le 10/06/2004 thierry*/
	 }
     }
   
   for(m=0; m<n; m++) niveau0+=h[m];
   niveau0=niveau0/n;
   for(m=0; m<n; m++) h[m]-=niveau0;     /* on retire le niveau moyen du residu*/
   step++;

   /* criteres de precison */
/*-----------------------------------------------------*/
   spmoy=0;
   for (m=0;m<n;m++) spmoy+=sealevel[m]-mean;
   spmoy/=n;
   predicmoy=0;
   for (m=0;m<n;m++) predicmoy+=predic[m];
   predicmoy/=n;
   crit=sqrt(spmoy*spmoy - (predicmoy)*(predicmoy));
   niveau0=0;
/*-----------------------------------------------------*/
    
  }/*while step*/
/*fin de lanalyse iterative*/
/*-----------------------------------------------------*/


/*--------------------------------*/
/*    edition des resultats       */
/*--------------------------------*/

  for(k=0; k < s.n; k++)
    {
      tabZ_sortie[k].reel=zr[k];
      tabZ_sortie[k].imag=zi[k];
    }

  


  free((double *)A);
  free((double *)b);
  free(pivot);
  for(k=0; k < s.n; k++) free((double *)cs[k]);
  for(k=0; k < s.n; k++) free((double *)sn[k]);
  free(cs);
  free(sn);
  free(zr);
  free(zi);
  free(e);
  free(predic);

  gsl_vector_free(B);
  gsl_vector_free(X);
  gsl_matrix_free(M);
  gsl_permutation_free(P);

}



/*---------------------------------------------------------------*/

double Tide_prediction(double time,tidal_wave wave,fcomplex Z,int verbose,int CTO)

/*---------------------------------------------------------------*/
/* The prediction is made from a true complex,                   */
/*      Z  =     (A cos(-G), A sin(-G) )                         */
/*---------------------------------------------------------------*/
{
  double V,f,ret;
  astro_ang_struct astro_ang;

 init_argument(time,verbose,CTO,&astro_ang);
 V=greenwhich_argument(wave,&astro_ang)+nodal_phase(wave,&astro_ang);
 f=nodal_factort(wave.formula,&astro_ang);

 ret=f*(cos(V)*Z.reel-sin(V)*Z.imag);
 return(ret); 
}


	
/*--------------------------------------------------------------------------------*/

void tidal_potential(tidal_wave wave,double lat,double lon, double *Amp, double *G,int terrestre)

/*--------------------------------------------------------------------------------*/
{
  float k2=0.3,h2=0.6;
  double C,S,a,dV,pi,dtr;

  pi=acos(-1.0);
  dtr=pi/180.0;

  if (terrestre) a=wave.Ap*h2;
  else a=wave.Ap*(1+k2-h2);
  switch (wave.nT)
    {
    case (0):

/*######################### Long period tide #########################

   potential/g = A*(1/2 -3/2 sin^2(Y)*cos(w*t+V0)

   dP/dx=  0
   dP/dy= -3*A*cos(Y)sin(Y)*cos(w*t+V0)

----------------------------------------------------------------------*/
      dV=0.0;
      C=cos(lat*dtr);
      S=sin(lat*dtr);
      *Amp = a*(0.5-1.5*S*S)/100;
      *G   = -dV*dtr;
    break;

    case (1):

/*########################### Diurnal tide ###########################

   potential/g = A*sin(2Y)*cos(w*t+V0+X)

----------------------------------------------------------------------*/

      dV=lon;
      C=cos(lat*dtr);
      S=sin(lat*dtr);
      *Amp = 2*a*S*C/100;
      *G   = -dV*dtr;
    break;

    case (2):

/*######################### Semi-diurnal tide #########################

   potential/g = A*cos^2(Y)*cos(w*t+V0+2*X)

----------------------------------------------------------------------*/

      dV=2*lon;
      C=cos(lat*dtr);
      S=sin(lat*dtr);
      *Amp = a*C*C/100;
      *G   = -dV*dtr;
    break;

/*####################### non-astronomical tide #######################
   potential/g = 0

----------------------------------------------------------------------*/
    default:
    break;

  }/*end switch */
}/* end*/


/*XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX*/

  void orthotide_analysis(double *sealevel,double *time, int n,wavelist s,int nopt, double lon, double lat, fcomplex *tabZ_sortie, int CTO)

/*XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX*/

{
  int k,l,m,neq,status,job,cnt,cst;
  int verbose=0,info; 
  int *pivot,toto;
  
  gsl_matrix *M;
  gsl_permutation *Per;
  gsl_vector *B,*X;
  int signum; 


  FILE *matrice;
  char carac[256];
  
  double V,d2r,f,tau;
  double *zr,*zi,mean,*h;
  double *A,*b;
  double Ap,Gp,**P;
  double tmp;

  tidal_wave wave;
  astro_ang_struct astro_ang;

  /*alloue en dur a douze(2*6inconnues)*/
  neq=12;


/*allocation*/  
/*-----------------------------------------------------------------*/
  /*alloue en dur a douze(2*6inconnues)*/

  P=(double **) malloc(neq*sizeof(double));
  for(k=0; k < neq; k++)  P[k]=(double *)malloc(n*sizeof(double));
  h=(double *)malloc(n*sizeof(double));
  A= (double *)malloc(neq*neq*sizeof(double));
  b= (double *)malloc(neq*sizeof(double));
  zr=(double *) malloc(s.n*sizeof(double));
  zi=(double *) malloc(s.n*sizeof(double));
  pivot=(int *)malloc(neq*sizeof(int));
  
M=gsl_matrix_calloc(neq,neq);
  X=gsl_vector_calloc(neq);
  B=gsl_vector_calloc(neq);
  Per=gsl_permutation_calloc(neq);

 
/*-----------------------------------------------------------------*/



  /*initialisation*/
/*-----------------------------------------------------------------*/
  d2r=pi/180.0;
  mean=0.;
  tau=2*24/*2jours*/;

  for(k=0; k < neq; k++)for(l=0; l<n; l++)P[k][l]=0;

  for(k=0; k<neq; k++) for(l=0; l<neq; l++) A[k*neq+l]=0;

  for(k=0; k<s.n; k++) zr[k]=0;
  for(k=0; k<s.n; k++) zi[k]=0;

  for(k=0; k<neq; k++) b[k]=0;
/*-----------------------------------------------------------------*/

/*niveau moyen*/
/*-----------------------------------------------------------------*/
  for(m=0; m<n; m++)
    {
    h[m]=sealevel[m];
    mean+=h[m];
    }
  mean/=n;
  for(m=0; m<n; m++)
    {
    h[m]-=mean;
    }
/*-----------------------------------------------------------------*/


/*construction de P et Q*/
/*-----------------------------------------------------------------*/
  for(m=0; m<n; m++)
    {
      
      for(k=0; k < s.n; k++)
	{
	  wave=s.wave[k];
	  init_argument(time[m],verbose,CTO,&astro_ang);
	  f=nodal_factort(wave.formula,&astro_ang);
	  V=greenwhich_argument(wave,&astro_ang)+nodal_phase(wave,&astro_ang) ;
	  if (s.wave[k].nT==2) cst=6;
	  if (s.wave[k].nT==1) cst=0; 
	    {
	      P[0+cst][m]+=f*cos(V)*cos(tau*wave.omega*d2r)*s.wave[k].Ap;
	      P[1+cst][m]+=f*cos(V)*sin(tau*wave.omega*d2r)*s.wave[k].Ap;
	      P[2+cst][m]+=f*cos(V)*s.wave[k].Ap;
	      P[3+cst][m]+=f*sin(V)*cos(tau*wave.omega*d2r)*s.wave[k].Ap;
	      P[4+cst][m]+=f*sin(V)*sin(tau*wave.omega*d2r)*s.wave[k].Ap;
	      P[5+cst][m]+=f*sin(V)*s.wave[k].Ap;
	    }
	}

    }
/*-----------------------------------------------------------------*/
	  
/*construction de matrice A*/
/*-----------------------------------------------------------------*/
  for(k=0; k < neq; k++) for(l=0; l<neq; l++) for(m=0; m<n; m++) A[k*neq+l]+=P[k][m]*P[l][m];
/*-----------------------------------------------------------------*/
/*construction de matrice M*/
/*-----------------------------------------------------------------*/
  for(k=0; k < neq; k++) for(l=0; l<neq; l++) for(m=0; m<n; m++) gsl_matrix_set(M,k,l,A[k*neq+l]);
/*-----------------------------------------------------------------*/


/*ecriture de matrice A*/
/*-----------------------------------------------------------------*/
/*  sprintf(carac,"./res/m.dat.%d",nopt);
  matrice=fopen(carac,"w");
  fprintf(matrice,"coordonnes: %7.4f  %7.4f \n",lon, lat );
 fprintf(matrice,"\n");
  for(k=0;k<neq;k++)
    {
      for(l=0;l<neq;l++)
	{
	  fprintf(matrice,"%8.4f ",A[l+k*neq]);
	}
      fprintf(matrice,"\n");
     
    }
  fclose(matrice);*/
/*-----------------------------------------------------------------*/


/*bidouille contruction de la matrice
  diagonale avec des 1*/
/*-----------------------------------------------------------------*/
  for(k=0; k <neq ; k++) 
    {
      tmp=gsl_matrix_get(M,k,k);
      if (tmp==0) gsl_matrix_set(M,k,k,1);
    }

/*factorisation de A*/
/*-----------------------------------------------------------------*/
#ifdef LINPACKF_
  dchdc_(A,&neq,&neq, work,pivot,&status, &info);
#elif ESSL
  dgef  (A,&neq,&neq,pivot,&status);
#elif LINPACKF
  dgefa (A,&neq,&neq,pivot,&status);
#elif LINPACKC
  dgefa (A,neq,neq,pivot,&status);
#elif GSL
  toto=gsl_linalg_LU_decomp(M,Per,&signum);
#endif
/*-----------------------------------------------------------------*/

/*construction second membre*/
/*-----------------------------------------------------------------*/
    for(k=0; k <neq; k++) for(m=0; m<n; m++) b[k]  +=P[k][m]*h[m]; 
    for(k=0; k <neq; k++) gsl_vector_set(B,k,b[k]);
   
/*-----------------------------------------------------------------*/

/*resolution de Ax=B*/
/*-----------------------------------------------------------------*/
job=0;
#ifdef LINPACKF_
    dgesl_ (A,&neq,&neq,pivot,b,&job);
#elif ESSL
    dges   (A,&neq,&neq,pivot,b);
#elif LINPACKF
    dgesl (A,&neq,&neq,pivot,b,&job);
#elif LINPACKC
   dgesl  (A,neq,neq,pivot,b,job);
#elif GSL
   toto=gsl_linalg_LU_solve(M,Per,B,X);
#endif
/*-----------------------------------------------------------------*/
for(k=0; k <neq; k++) b[k]=gsl_vector_get(X,k);
/*-----------------------------------------------------------------*/

 
/*--------------------------------*/
/*    edition des resultats       */
/*--------------------------------*/

  for(k=0; k < s.n; k++)
    {
      if (s.wave[k].nT==1) 
	{
	  zr[k]=(b[0]*cos(tau*s.wave[k].omega*d2r)+b[1]*sin(tau*s.wave[k].omega*d2r)+b[2])*s.wave[k].Ap;
	  zi[k]=(b[3]*cos(tau*s.wave[k].omega*d2r)+b[4]*sin(tau*s.wave[k].omega*d2r)+b[5])*s.wave[k].Ap;
	  tabZ_sortie[k].reel=zr[k];
	  tabZ_sortie[k].imag=zi[k];
	}

      else if (s.wave[k].nT==2) 
	{
	  zr[k]=(b[6]*cos(tau*s.wave[k].omega*d2r)+b[7]*sin(tau*s.wave[k].omega*d2r)+b[8])*s.wave[k].Ap;
	  zi[k]=(b[9]*cos(tau*s.wave[k].omega*d2r)+b[10]*sin(tau*s.wave[k].omega*d2r)+b[11])*s.wave[k].Ap;
	  tabZ_sortie[k].reel=zr[k];
	  tabZ_sortie[k].imag=zi[k];
	}

    }

  

  for(k=0; k <neq; k++) free((double *)P[k]);
  free(P);
  free((double *)A);
  free((double *)b);
  free(pivot);
  free(zr);
  free(zi);
  free(h);
  gsl_vector_free(B);
  gsl_vector_free(X);
  gsl_matrix_free(M);
  gsl_permutation_free(Per);

}


/*---------------------------------------------------------------------------------------------------*/

 void mixte_analysis(int nmes,double *sealevel,double *time,wavelist s, fcomplex *tabZ_sortie, int *KEEP,typeA typ, int nb_group, int CTO)
 {
/*---------------------------------------------------------------------------------------------------*/
   int k,l,m,code,ci;
   int toto,signum;  
   gsl_matrix *MATRICE;
   gsl_permutation *PER;
   gsl_vector *B,*X; 
 
   double **cs,**sn;
   double *A,*M;
   double **P,*b,*h;
   double V,f,mean;
   int typ_h2=typ.h*2;
   int typ_o6=6*2;
   int typ_sum=typ_h2+typ_o6;
   double Ap1,kappa,tmp,dooble;

   int tau=2*24;/*2jours*/
  
   double d2r=pi/180.0;
  FILE *matrice;
  int cnt,i;
  
  char carac[256];
  astro_ang_struct astro_ang;

/*-----------------------------------------------------*/
   /* allocation */
/*-----------------------------------------------------*/
  h=(double *)malloc(nmes*sizeof(double));
  b=(double *) malloc(typ_sum*sizeof(double));
  cs=(double **) malloc(typ.h*sizeof(double));
  sn=(double **) malloc(typ.h*sizeof(double));
  for(k=0; k < typ.h; k++) cs[k]=(double *) malloc(nmes*sizeof(double));
  for(k=0; k < typ.h; k++) sn[k]=(double *) malloc(nmes*sizeof(double));
  A= (double *)malloc(typ_h2*typ_h2*sizeof(double));
  M= (double *)malloc(typ_o6*typ_o6*sizeof(double));
  PER=gsl_permutation_calloc(typ_sum);
  B=gsl_vector_calloc(typ_sum);
  X=gsl_vector_calloc(typ_sum);
  MATRICE=gsl_matrix_calloc(typ_sum,typ_sum);

  P=(double **) malloc(typ_o6*sizeof(double));
  for(k=0; k <typ_o6 ; k++)  P[k]=(double *)malloc(nmes*sizeof(double));

/*-----------------------------------------------------*/

  /* initialisation */
/*-----------------------------------------------------*/
  for(k=0; k<typ_sum; k++) b[k]=0;
  for(k=0; k<typ_h2; k++)for(l=0; l<typ_h2; l++) A[k*typ_h2+l]=0;
 
  for(k=0; k < typ_o6; k++)for(l=0; l<nmes; l++)P[k][l]=0;

  for(k=0; k<typ_o6; k++) for(l=0; l<typ_o6; l++) M[k*typ_o6+l]=0;

/*-----------------------------------------------------*/
/*niveau moyen*/
/*-----------------------------------------------------------------*/
   mean=0.;
 for(m=0; m<nmes; m++)
    {
    h[m]=sealevel[m];
    mean+=h[m];
    }
  mean/=nmes;
  for(m=0; m<nmes; m++)
    {
    h[m]-=mean;
    }
/*-----------------------------------------------------------------*/
  /* sin and cos building */
/*-----------------------------------------------------*/
  for(k=0; k <typ.h ; k++)
    {
    for(m=0; m<nmes; m++)
      {
	init_argument(time[m],0,CTO,&astro_ang);
     	f=nodal_factort(s.wave[k].formula,&astro_ang);
	V=greenwhich_argument(s.wave[k],&astro_ang)+nodal_phase(s.wave[k],&astro_ang);
       cs[k][m]=f*cos(V);
       sn[k][m]=f*sin(V);
      
     }
    }
/*-----------------------------------------------------*/
  /* matrix A building */
/*-----------------------------------------------------*/
  for(k=0; k < typ.h; k++)
    {
    for(l=0; l<typ.h; l++) 
      {
      for(m=0; m<nmes; m++)
         {
	   A[2*l*typ_h2    +2*k]  +=cs[k][m]*cs[l][m];
	   A[2*l*typ_h2    +2*k+1]+=sn[k][m]*cs[l][m];
	   A[(2*l+1)*typ_h2+2*k]  +=cs[k][m]*sn[l][m];
	   A[(2*l+1)*typ_h2+2*k+1]+=sn[k][m]*sn[l][m];
	 }
      }
    }
/*-----------------------------------------------------*/


  /* admittance treatment */
/*-----------------------------------------------------*/
 for(k=0; k < typ.h; k++)
     if (KEEP[k]==1)
       {  
	 for(l=0; l<typ.h; l++) 
	   {
	     A[2*l*typ_h2    +2*k]  =0;
	     A[2*l*typ_h2    +2*k+1]=0;/*toutes la ligne a 0*/
	     A[(2*l+1)*typ_h2+2*k]  =0;
	     A[(2*l+1)*typ_h2+2*k+1]=0;
	   }
	 A[2*k*typ_h2+2*k]  =-1;      /*la diagonale a -1*/
	 A[(2*k+1)*typ_h2+2*k+1]  =-1;
	 
	 code=s.wave[k].admit;
	 ci=0;     
	 while(code!=s.wave[ci].code){ci++;}
	 Ap1=s.wave[ci].Ap;
	 kappa=s.wave[k].Ap/Ap1;
	 A[2*ci*typ_h2+2*k]  =kappa;  /*rapport des potential*/
	 A[(2*ci+1)*typ_h2+2*k+1]  =kappa;
       }/*if keep*/
/*-----------------------------------------------------*/

  /* affichage de la matrice*/
/*-----------------------------------------------------*/
/*  sprintf(carac,"./mmixte.dat.%d",nb_group); */
/*   matrice=fopen(carac,"w"); */
/*  fprintf(matrice,"\n"); */
/*   fprintf(matrice,"    "); */
/*   for(k=0;k<2*s.n;k++)  */
/*     {       */
/*       cnt=floor(k/2); */
/*       fprintf(matrice,"%5s",s.wave[cnt].name); */
/*     } */
/*   fprintf(matrice,"\n"); */
/*   for(k=0;k<s.n*2;k++) */
/*     { */
/*       cnt=floor(k/2); */
/*       fprintf(matrice,"%3s ",s.wave[cnt].name); */
/*       for(i=0;i<4*s.n*s.n;i+=2*s.n) */
/* 	{ */
/* 	  fprintf(matrice,"%4.0f ",A[i+k]); */
/* 	} */
/*       fprintf(matrice,"\n"); */
/*       fprintf(matrice,"----------------------------------------------------------------------"); */
/*       fprintf(matrice,"----------------------------------------------------------------------"); */
/*       fprintf(matrice,"----------------------------------\n"); */
/*     } */
/*   fclose(matrice); */
/*-----------------------------------------------------*/


/*CHANGEMENT DE METHODE*/
/*---------------------*/
 
/*-----------------------------------------------------*/
/*construction de P et Q*/
/*-----------------------------------------------------------------*/
  for(m=0; m<nmes; m++)
    {
      
      for(k=typ.h; k <(typ.h+typ.o); k++)
	{
	  init_argument(time[m],0,CTO,&astro_ang);
	  f=nodal_factort(s.wave[k].formula,&astro_ang);
	  V=greenwhich_argument(s.wave[k],&astro_ang)+nodal_phase(s.wave[k],&astro_ang) ;
	      if (s.wave[k].nT==2) ci=6;
	      if (s.wave[k].nT==1) ci=0; 
	  P[0+ci][m]+=f*cos(V)*cos(tau*s.wave[k].omega*d2r)*s.wave[k].Ap;
	  P[1+ci][m]+=f*cos(V)*sin(tau*s.wave[k].omega*d2r)*s.wave[k].Ap;
	  P[2+ci][m]+=f*cos(V)*s.wave[k].Ap;
	  P[3+ci][m]+=f*sin(V)*cos(tau*s.wave[k].omega*d2r)*s.wave[k].Ap;
	  P[4+ci][m]+=f*sin(V)*sin(tau*s.wave[k].omega*d2r)*s.wave[k].Ap;
	  P[5+ci][m]+=f*sin(V)*s.wave[k].Ap;
	}

    }
/*-----------------------------------------------------------------*/
	  
/*construction de matrice M*/
/*-----------------------------------------------------------------*/
  for(k=0; k < typ_o6; k++) for(l=0; l<typ_o6; l++) for(m=0; m<nmes; m++) M[k*typ_o6+l]+=P[k][m]*P[l][m];

/*-----------------------------------------------------------------*/
	  
/*-----------------------------------------------------------------*/

/*ASSEMBLAGE DES DEUX METHODES*/
/*----------------------------*/

/*construction de matrice MATRICE*/
/*-----------------------------------------------------------------*/

 
  for(k=0; k <typ_h2; k++) for(l=0; l<typ_h2;l++)  gsl_matrix_set(MATRICE,l,k,A[k*typ_h2+l]);

  for(k=typ_h2; k <typ_sum ; k++) for(l=typ_h2; l<typ_sum; l++)gsl_matrix_set(MATRICE,k,l,M[(k-typ_h2)*typ_o6+(l-typ_h2)]);


/*couplage entre les  methodes*/
/*-----------------------------------------------------------------*/

 for(l=typ_h2; l<(typ_h2+typ_o6); l++) for(k=0; k < typ.h; k++)
   {
     dooble=0;
     for(m=0; m<nmes; m++) dooble+=cs[k][m]*P[l-typ_h2][m];
     gsl_matrix_set(MATRICE,2*k,l,dooble);
     gsl_matrix_set(MATRICE,l,2*k,dooble);

     dooble=0;
     for(m=0; m<nmes; m++) dooble+=sn[k][m]*P[l-typ_h2][m];
     gsl_matrix_set(MATRICE,2*k+1,l,dooble);
     gsl_matrix_set(MATRICE,l,2*k+1,dooble);
   }



/*-----------------------------------------------------------------*/


/*bidouille contruction de la matrice
  diagonale avec des 1*/
/*-----------------------------------------------------------------*/
  for(k=0; k <typ_sum ; k++) 
    {
      tmp=gsl_matrix_get(MATRICE,k,k);
      if (tmp==0) gsl_matrix_set(MATRICE,k,k,1);
    }

/* Factorisation de la MATRICE*/
/*----------------------------*/

  toto=gsl_linalg_LU_decomp(MATRICE,PER,&signum);


/*--------------------------------*/
/*construction de B second membre */
/*--------------------------------*/

  for(k=0; k < typ.h; k++) for(m=0; m<nmes; m++) {b[2*k]+=cs[k][m]*h[m]; b[2*k+1]+=sn[k][m]*h[m];} /*construction second membre harmo*/
  for(k=0; k < typ.h; k++)if (KEEP[k]==1){b[2*k]=0; b[2*k+1]=0;} /*construction second membre admitance*/ 
  for(k=0; k <typ_o6; k++) for(m=0; m<nmes; m++) b[k+typ_h2]  +=P[k][m]*h[m];  /*construction second membre ortho*/  

  for(k=0; k <typ_sum; k++) gsl_vector_set(B,k,b[k]);
/*-----------------------------------------------------*/
/*resolution de Ax=B*/
/*-----------------------------------------------------------------*/
  toto=gsl_linalg_LU_solve(MATRICE,PER,B,X);

/*-----------------------------------------------------------------*/
  for(k=0; k <typ_sum; k++) b[k]=gsl_vector_get(X,k);
/*-----------------------------------------------------------------*/



 for(k=0; k <typ.h; k++)                /* iteration */
     {
       tabZ_sortie[k].reel+=b[2*k];
       tabZ_sortie[k].imag+=b[2*k+1];
     }

  for(k=typ.h; k < (typ.h+typ.o); k++)
    {
      if (s.wave[k].nT==1) 
	{
	  tabZ_sortie[k].reel=(b[0+typ_h2]*cos(tau*s.wave[k].omega*d2r)+b[1+typ_h2]*sin(tau*s.wave[k].omega*d2r)+b[2+typ_h2])*s.wave[k].Ap;
	  tabZ_sortie[k].imag=(b[3+typ_h2]*cos(tau*s.wave[k].omega*d2r)+b[4+typ_h2]*sin(tau*s.wave[k].omega*d2r)+b[5+typ_h2])*s.wave[k].Ap;
	}

      else if (s.wave[k].nT==2) 
	{
	  tabZ_sortie[k].reel=(b[6+typ_h2]*cos(tau*s.wave[k].omega*d2r)+b[7+typ_h2]*sin(tau*s.wave[k].omega*d2r)+b[8+typ_h2])*s.wave[k].Ap;
	  tabZ_sortie[k].imag=(b[9+typ_h2]*cos(tau*s.wave[k].omega*d2r)+b[10+typ_h2]*sin(tau*s.wave[k].omega*d2r)+b[11+typ_h2])*s.wave[k].Ap;
	}

    }








/*-----------------------------------------------------------------*/
  free(h);
  free(b);
  for(k=0; k < typ.h; k++) {free(cs[k]);free(sn[k]);}
  free(cs);
  free(sn);
  free(A);
  free(M);
  gsl_permutation_free(PER);
  gsl_vector_free(B);
  gsl_vector_free(X);
  gsl_matrix_free(MATRICE);
  for(k=0; k <typ_o6 ; k++)  free(P[k]);
  free(P);


/*-----------------------------------------------------*/


 }/*end.*/


/*-----------------------------------------------------------------------*/

void calendary(int njd,date_t *actual)

/*-----------------------------------------------------------------------*/

{
  int njul,nj,nb,nm1,nj3,m,j,ndj;
  int na,nm,nd;
  int n[12]= {31,28,31,30,31,30,31,31,30,31,30,31};

/*-----------------------------------------------------------------------
! Input = njd: nombre ecoules depuis le 1er  Janvier 1950, 0 heure
! Output= nd,nm,na: jour, mois annee calendaire
!-----------------------------------------------------------------------*/

  njul=njd+1;
  na=njul/365;
  nj=njul-na*365;
  nb=(na+1)/ 4;
  nj=nj-nb;

  if (nj >  0) goto a1;

  na=na+1949;
  nm=12;
  nd=nj+31;
  goto a9000;

  a1:
  j=na-2-nb*4;
  na=na+1950;

  if (j < 0) goto a5000;

  a4000: 
  if (60-nj < 0)  goto  a4500;
  if (60-nj == 0) goto  a7000;
  goto a5000;

  a4500: 
  nm1=60;

  m=3;
  goto a6000;

  a5000: 
  nm1=0;

  m=1;

  a6000: 
  ndj=nm1+n[m-1];

  nj3=nj-ndj;

  if (nj3 <= 0) goto a8000;

  a6500: 
  m=m+1;

  nm1=ndj;
  goto a6000;

  a7000: 
  nm=2;

  nd=29;

  goto a9000;

  a8000: 
  nm=m;

  nd=nj-nm1;

  a9000: 
  actual->year=na;
  actual->month=nm;
  actual->day=nd;
  actual->second=0;

}

/*-------------------------------------------*/

void getcnesdate(double t,date_t *actual)

/*-------------------------------------------*/
{
  int N,nday;
  float second;

/* t is time elapsed from 1/1/1950 in hours */
  
  nday=floor(t/24.0);

  calendary(nday,actual);
  second=(t-nday*24)*3600.;
  actual->second=second;
}

/*XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX*/

  void harmonic_analysis_parra(double *sealevel,double *time, int n,wavelist s,int nopt, double lon, double lat, fcomplex *tabZ_sortie, double *h, int *KEEP,int CTO,int CPU)

/*XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX*/
/* ---------------------------------------------------------------------------
  
   Makes an harmonic analysis from a classical harmonic formalism;
   returns amplitude a and phase lag G from a sealevel time serie and for a specified wave list;
   !! Need a KEEP array specifying wave(s) that will be deduced by admittance 

-------------------------------------------------------------------------------*/
{
  int i,k,l,m,neq,status,job,step,nw,cnt;
  int *pivot;
  int info;
  int ldl,ldr;
  int stepmax=300; 
  int verbose=0; 
  int code,ci,toto;
     
  gsl_matrix *M;
  gsl_permutation *P;
  gsl_vector *B,*X;
  int signum; 
  int vrai,nbthreads,debut,fin;

  double *A,*b;
  double **cs,**sn;
  double V,V0,dph2rps,omega,f,niveau0=0;
  double *zr,*zi,mean;
  double  *svals;
  double  *e, *lsvec, *rsvec;
  double *predic,crit,spmoy,predicmoy;
  double kappa,Ap1;

  FILE *matrice;
  
  char carac[256];

  tidal_wave wave;
  astro_ang_struct astro_ang;
 
  /*parra*/
  P1_struct *P1;
  P2_struct *P2;
  pthread_t *tid;   
 

  /*premiere ligne du code */
  neq=2*s.n;



   /* allocation */
/*-----------------------------------------------------*/
  predic=(double *) calloc(n,sizeof(double)); /*modif le 10/06/2004 Thierry de malloc a calloc*/

  	
  M=gsl_matrix_calloc(neq,neq);
  X=gsl_vector_calloc(neq);
  B=gsl_vector_calloc(neq);
  P=gsl_permutation_calloc(neq);


  A= (double *)malloc(neq*neq*sizeof(double));
  b= (double *)malloc(neq*sizeof(double));
  zr=(double *) malloc(s.n*sizeof(double));
  zi=(double *) malloc(s.n*sizeof(double));

  cs=(double **) malloc(s.n*sizeof(double));
  sn=(double **) malloc(s.n*sizeof(double));
  for(k=0; k < s.n; k++) cs[k]=(double *) malloc(n*sizeof(double));
  for(k=0; k < s.n; k++) sn[k]=(double *) malloc(n*sizeof(double));
  pivot=(int *)malloc(neq*sizeof(int));
  e=(double *)malloc(neq*sizeof(double));
/*-----------------------------------------------------*/

  /* mean level retiring */
/*-----------------------------------------------------*/
  mean=0.;
  for(m=0; m<n; m++)
    {
    h[m]=sealevel[m];
    mean+=h[m];
    }
  mean/=n;
  for(m=0; m<n; m++)
    {
    h[m]-=mean;
    }
/*-----------------------------------------------------*/

  /* initialisation */
/*-----------------------------------------------------*/

  dph2rps=pi/(180.0*3600.0);

  nw=s.n; 
  for(k=0; k<neq; k++)for(l=0; l<neq; l++) A[k*neq+l]=0;

  for(k=0; k<nw; k++) zr[k]=0;
  for(k=0; k<nw; k++) zi[k]=0;
  for(k=0; k<s.n; k++) b[k]=0;
/*-----------------------------------------------------*/




  /* sin and cos building */
/*-----------------------------------------------------*/
  P1=calloc(s.n,sizeof(P1_struct));
  printf("threads\n");
 tid=malloc(s.n*sizeof(pthread_t)); 

 vrai=1;
 debut=0;
 fin=CPU;
 if(fin>s.n)fin=s.n;
 while(vrai==1)
   {
     for(k=debut;k<fin;k++)
       { 
	 P1[k].time=time;
	 P1[k].verbose=verbose;
	 P1[k].CTO=CTO;
	 P1[k].wave=&s.wave[k];
	 P1[k].cs=cs[k];
	 P1[k].sn=sn[k];
	 P1[k].n=n;
	 pthread_create(&tid[k], NULL,calcul_cos_sin,(void *)&(P1[k]));
       }	
     for (k=debut;k<fin;k++)pthread_join(tid[k], NULL);
     debut=fin;
     if(fin==s.n)vrai=99;
     fin+=CPU;
     if(fin>s.n)fin=s.n;

   }
/*-----------------------------------------------------*/


/*-----------------------------------------------------*/
  /* matrix A building */
/*-----------------------------------------------------*/
  P2=calloc(s.n,sizeof(P2_struct));

 vrai=1;
 debut=0;
 fin=CPU;
 if(fin>s.n)fin=s.n;
 while(vrai==1)
   {
     for(k=debut; k < fin; k++)
       {
	 P2[k].cs=cs;
	 P2[k].sn=sn;
	 for(l=0; l<s.n; l++)
	   {
	     P2[k].cs[l]=cs[l];
	     P2[k].sn[l]=sn[l];
	   }
	 P2[k].n=n;
	 P2[k].neq=neq;
	 P2[k].Sn=s.n;
	 P2[k].k=k;
	 P2[k].A=A;
	 pthread_create(&tid[k], NULL,calcul_A,(void *)&(P2[k]));
       }
     for (k=debut;k<fin;k++)pthread_join(tid[k], NULL);
     debut=fin;
     if(fin==s.n)vrai=99;
     fin+=CPU;
     if(fin>s.n)fin=s.n;
     

   }
  /*-----------------------------------------------------*/


  /* admittance processing */
/*-----------------------------------------------------*/
 for(k=0; k < s.n; k++)
     if (KEEP[k]==1)
       {  
	 for(l=0; l<s.n; l++) 
	   {
	     A[2*l*neq    +2*k]  =0;
	     A[2*l*neq    +2*k+1]=0;/*toutes la ligne a 0*/
	     A[(2*l+1)*neq+2*k]  =0;
	     A[(2*l+1)*neq+2*k+1]=0;
	   }
	 A[2*k*neq+2*k]  =-1;      /*la diagonale a -1*/
	 A[(2*k+1)*neq+2*k+1]  =-1;
	 
	 code=s.wave[k].admit;
	 ci=0;     
	 while(code!=s.wave[ci].code){ci++;/*if(ci>=s.n)break;*/}
	 Ap1=s.wave[ci].Ap;
	 kappa=s.wave[k].Ap/Ap1;
	 A[2*ci*neq+2*k]  =kappa;  /*rapport des potential*/
	 A[(2*ci+1)*neq+2*k+1]  =kappa;
       }/*if keep*/
/*-----------------------------------------------------*/

/*-----------------------------------------------------*/
  /* matrix M building */
/*-----------------------------------------------------*/
  for(k=0; k <neq; k++) for(l=0; l<neq;l++)  gsl_matrix_set(M,l,k,A[k*neq+l]);
 
/*-----------------------------------------------------*/




  /* affichage de la matrice*/
/*-----------------------------------------------------*/
/*  sprintf(carac,"./mharmo.dat.%d",nopt); */
/*   matrice=fopen(carac,"w"); */
/*   fprintf(matrice,"coordonnes: %7.4f  %7.4f \n",lon, lat ); */
/*  fprintf(matrice,"\n"); */
/*   fprintf(matrice,"    "); */
/*   for(k=0;k<2*s.n;k++)  */
/*     {       */
/*       cnt=floor(k/2); */
/*       fprintf(matrice,"%5s",s.wave[cnt].name); */
/*     } */
/*   fprintf(matrice,"\n"); */
/*   for(k=0;k<s.n*2;k++) */
/*     { */
/*       cnt=floor(k/2); */
/*       fprintf(matrice,"%3s ",s.wave[cnt].name); */
/*       for(i=0;i<4*s.n*s.n;i+=2*s.n) */
/* 	{ */
/* 	  fprintf(matrice,"%4.0f ",A[i+k]); */
/* 	} */
/*       fprintf(matrice,"\n"); */
/*       fprintf(matrice,"----------------------------------------------------------------------"); */
/*       fprintf(matrice,"----------------------------------------------------------------------"); */
/*       fprintf(matrice,"----------------------------------\n"); */
/*     } */
/*   fclose(matrice); */
/*-----------------------------------------------------*/

  /* affichage des vecteurs propres */
/*-----------------------------------------------------*/

 /*  lsvec=malloc(neq*neq*sizeof(double)); */
/*   rsvec=malloc(neq*neq*sizeof(double)); */

/*   status=1; */
/*   dsvdc(A, neq, neq, neq, svals, e, lsvec, ldl=neq, rsvec,ldr=neq, job=11, &info); */


/*   sprintf(carac,"valp.dat.%d",nopt); */
/*   matrice=fopen(carac,"w"); */
/*   for(i=0;i<neq;i++)  fprintf(matrice," %lf  \n",svals[i]); */
/*   fclose(matrice); */
/*-----------------------------------------------------*/

  /* matrix A factorization */
/*-----------------------------------------------------*/
#ifdef LINPACKF_
  dchdc_(A,&neq,&neq, work,pivot,&status, &info);
#elif ESSL
  dgef  (A,&neq,&neq,pivot,&status);
#elif LINPACKF
  dgefa (A,&neq,&neq,pivot,&status);
#elif LINPACKC
  dgefa (A,neq,neq,pivot,&status);
#elif GSL
  toto=gsl_linalg_LU_decomp(M,P,&signum);
#endif
/*-----------------------------------------------------*/


/*--------------------------------*/
/*   analyse iterative            */
/*--------------------------------*/
/*  gsl_matrix_transpose(M);*/
crit=1;
step=0;
while((crit > 1.e-25)&&(step <= stepmax))
  {
    for(k=0; k<neq; k++) b[k]=0;
    for(k=0; k < s.n; k++)
      {
	for(m=0; m<n; m++)
	  {
	    b[2*k]  +=cs[k][m]*h[m];      /*construction second membre*/
	    b[2*k+1]+=sn[k][m]*h[m];
	  }
      }
for(k=0; k < s.n; k++)
     if (KEEP[k]==1)
       {  
	    b[2*k]  =0;      /*construction second membre admitance*/
	    b[2*k+1]=0;
       }
/*-----------------------------------------------------*/

/*--------------------------------*/
/*construction de B second membre */
/*--------------------------------*/

for(k=0; k <neq; k++) gsl_vector_set(B,k,b[k]);
/*-----------------------------------------------------*/




  /* resolution Ax=b */
/*-----------------------------------------------------*/
    job=0;
#ifdef LINPACKF_
    dgesl_ (A,&neq,&neq,pivot,b,&job);
#elif ESSL
    dges   (A,&neq,&neq,pivot,b);
#elif LINPACKF
    dgesl (A,&neq,&neq,pivot,b,&job);
#elif LINPACKC
   dgesl  (A,neq,neq,pivot,b,job);
#elif GSL
   toto=gsl_linalg_LU_solve(M,P,B,X);
#endif
/*-----------------------------------------------------*/


for(k=0; k <neq; k++) b[k]=gsl_vector_get(X,k);


   for(k=0; k < s.n; k++)                /* iteration */
     {
       zr[k]+=b[2*k];
       zi[k]+=b[2*k+1];
     }
   for(m=0; m<n; m++)
     {
       for(k=0; k < s.n; k++)
	 {
	   h[m]-=cs[k][m]*b[2*k]+sn[k][m]*b[2*k+1];      /* residu*/
/* 	   predic[m]=cs[k][m]*b[2*k]+sn[k][m]*b[2*k+1];   */
	   predic[m]+=cs[k][m]*b[2*k]+sn[k][m]*b[2*k+1];  /*modif le 10/06/2004 thierry*/
	 }
     }
   
   for(m=0; m<n; m++) niveau0+=h[m];
   niveau0=niveau0/n;
   for(m=0; m<n; m++) h[m]-=niveau0;     /* on retire le niveau moyen du residu*/
   step++;

   /* criteres de precison */
/*-----------------------------------------------------*/
   spmoy=0;
   for (m=0;m<n;m++) spmoy+=sealevel[m]-mean;
   spmoy/=n;
   predicmoy=0;
   for (m=0;m<n;m++) predicmoy+=predic[m];
   predicmoy/=n;
   crit=sqrt(spmoy*spmoy - (predicmoy)*(predicmoy));
   niveau0=0;
/*-----------------------------------------------------*/
    
  }/*while step*/
/*fin de lanalyse iterative*/
/*-----------------------------------------------------*/


/*--------------------------------*/
/*    edition des resultats       */
/*--------------------------------*/

  for(k=0; k < s.n; k++)
    {
      tabZ_sortie[k].reel=zr[k];
      tabZ_sortie[k].imag=zi[k];
    }

  


  free((double *)A);
  free((double *)b);
  free(pivot);
  for(k=0; k < s.n; k++) free((double *)cs[k]);
  for(k=0; k < s.n; k++) free((double *)sn[k]);
  free(cs);
  free(sn);
  free(zr);
  free(zi);
  free(e);
  free(predic);

  gsl_vector_free(B);
  gsl_vector_free(X);
  gsl_matrix_free(M);
  gsl_permutation_free(P);

}



void * calcul_cos_sin(void *input)

{

  P1_struct *P1;
  int m;
  double f,V;

  P1=(P1_struct *)input;
  
  P1->astro_ang=calloc(1,sizeof(astro_ang_struct));

  for(m=0; m<P1->n; m++)
    {
      init_argument(P1->time[m],P1->verbose,P1->CTO,P1->astro_ang);
      f=nodal_factort(P1->wave->formula,P1->astro_ang);
      V=greenwhich_argument(*P1->wave,P1->astro_ang)+nodal_phase(*P1->wave,P1->astro_ang);
     
      P1->cs[m]=f*cos(V);
      P1->sn[m]=f*sin(V);
    }

  free(P1->astro_ang);

}

void * calcul_A(void *input)
{
  P2_struct *P2;
  int m,l,k,neq;
  
  P2=(P2_struct *)input;
  k=P2->k;
  neq=P2->neq;

  for(l=0; l<P2->Sn; l++) 
    {
      for(m=0; m<P2->n; m++)   
	{
	  P2->A[2*l*neq    +2*k]  +=P2->cs[k][m]*P2->cs[l][m];
	  P2->A[2*l*neq    +2*k+1]+=P2->sn[k][m]*P2->cs[l][m];
	  P2->A[(2*l+1)*neq+2*k]  +=P2->cs[k][m]*P2->sn[l][m];
	  P2->A[(2*l+1)*neq+2*k+1]+=P2->sn[k][m]*P2->sn[l][m];
	}
    }
}
