
#define MAIN_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <ctype.h>
#include <string.h>   
#include <strings.h>
#include <errno.h>
#include <time.h>

#include "analyse_alti.h"
#include "aktarus.h"

#include "gsl/gsl_cblas.h"
#include "gsl/gsl_linalg.h"
#include "gsl/gsl_vector.h"
#include "gsl/gsl_matrix.h"
#include "gsl/gsl_permutation.h"
#include "gsl/gsl_statistics_double.h"



/*---------------------------------------------------------------------------*/
/*                            main program                                   */ 
/*                            ------------                                   */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* use:    analyseX.XX.exe ini.hut filename                                  */ 
/*                            ------------                                   */
/*---------------------------------------------------------------------------*/

int main (int argc, char *argv[])
{



  /*---------------------------------------------------------------------------*/
  /*    declarations                                                           */
  /*---------------------------------------------------------------------------*/

  char input[256],a;
  char stationname[20];
  char stryear[20],strmonth[20],strday[20],strhalf[20];
  char flpath[256]="/data/ocean/produits/p1/from_nivmer/loading/FES99/deformation/";        /*   path FES load effect -- !!! idem*/
  /* non valide     voir top ecmwf      char ppath[256]="../DATA/PRESSION";     /*  path IB model*/ 

  double *sealevel,*datatime,t1,t2;
  double lat,lon,trecord;
  double Echarge; 
  double mean=0;
  double *sealevelcopy=NULL,*ibcopy=NULL,*eccopy=NULL,*residual=NULL,*S_tide=NULL;
  double Ap,Gp;
  double Hmask;
  double rms1,rms2;
  double *noise;

  int nopt,nmes,n,i,k,flag_error,cnt,clean;
  int ib_mean=0;
  int filtre,cedex,sampling,rstatus;
  int hour, delta_t1,delta_t2,*tup,trans;
  int terrestre,*KEEP;
  int *position;
  int nb_group;/*nb de groupe d onde orthotide*/
  int old_nA;
  int firstyear,numstation; /* hawaii format*/
  int startyear,endyear;
  int prin_cnt=0;
  conf_struct conf;

  typeA nA;


  FILE *error,*res_dat; 
  FILE *fic_data,*sys_tmp,*fft_file;
  FILE *spectrefile;

  wavelist reference_s,work_s,mixteLIST,mixtework;

  float *amplitude,*retard;
  float *buf1, *buf2, masque;

  fcomplex Z, tabZ[34], *tabZ_sortie,mask;

  tidal_wave wave;

  grid_t grid;

  date_t actual,firstdate;

  time_t time1,time2;

  /*---------------------------------------------------------------------------*/
  /* initialisations                                                           */
  /*---------------------------------------------------------------------------*/   
  if(argc!=3) 
    {
      perror("wrong run parameters");
      printf("\nuse: analyseX.XX.exe ini.hut filename ");    
      exit(1);
    }
  
 sprintf(conf.data_filename,"%s",argv[2]);
 sprintf(conf.program_name,"%s",argv[0]);

  nopt=0;
  cnt=0;
  for (i=0;i<34;i++) {tabZ[i].reel=0.;tabZ[i].imag=0.;}
  
  fic_data=NULL; 
  fic_data=fopen(conf.data_filename,"r");
  if (fic_data==NULL) {printf("can't find data file : %s \n",conf.data_filename),exit(1);};
  
  question(argv[1],&conf);
  
  
  reference_s.wave=(tidal_wave *)malloc(conf.nbond*sizeof(tidal_wave)); 
  tup=(int *)malloc(conf.nbond*sizeof(int));
  /*---------------------------------------------------------------------------*/
  /*      spectrum loading                                                     */
  /*---------------------------------------------------------------------------*/
  
  tide_wavearray_initC(&reference_s,conf.nbond );
  
  
  /*---------------------------------------------------------------------------*/
  /*     sealevel time series loading from data file                           */
  /*---------------------------------------------------------------------------*/
  
  if(conf.format=='l') conf.nXpoints=header(fic_data);
  else conf.nXpoints=1;
  
  tabZ_sortie=(fcomplex *)malloc(reference_s.n*sizeof(fcomplex));
  datatime=(double *)malloc(conf.nbdata*sizeof(double));
  sealevel=(double *)malloc(conf.nbdata*sizeof(double));
  work_s.wave=(tidal_wave *)malloc(conf.nbond*sizeof(tidal_wave));
  
  conf.mgr_file=NULL;
  conf.sum_file=NULL;
 
  interactive(&conf);
 
  KEEP=(int *)malloc(reference_s.n*sizeof(int));
  
  if (conf.analys_type==2)
    {
      mixteLIST.wave=(tidal_wave *)malloc(conf.nbond*sizeof(tidal_wave)); 
      mixtework.wave=(tidal_wave *)malloc(conf.nbond*sizeof(tidal_wave)); 
      position=(int *)malloc(reference_s.n*sizeof(int));
      nb_group=questionmixte(position, reference_s, &nA, &mixteLIST);
      mixteLIST.n=conf.nbond;
    }
  old_nA=0;



  /*---------------------------------------------------------------------------*/
  /*    loop on crossover points                                               */
  /*---------------------------------------------------------------------------*/
  while (nopt<conf.nXpoints)
    { 
      flag_error=0;
      cedex=0;
      nopt++;
      prin_cnt++;
    
  
      /*-----------------------------------------------*/
      /* data loading from different formats data file */
      /*-----------------------------------------------*/
      switch (conf.format)
	{
	case 'l':
	  {
	    /*---------- .list format -----------*/
	    nmes=load_serie(datatime, sealevel,&lon,&lat,fic_data,conf.time_unit);
	    if(conf.nXpoints<20000)
	      {
		printf("\nLoading the data file ...\n");
		printf("%d valid data\n",nmes);
	      }
	    sprintf(stationname,"station");
	    numstation=000;
	    break;
	  };

	case 'h': 
	  {
	    /*------- .dat (GLOSS) format------- */
	    printf("\nLoading the data file ...\n");

	    fscanf(fic_data,"%3d%8c%4d",&numstation,stationname,&firstdate.year);
	    do {a=fgetc(fic_data);} while ( a!= '\n');
	    fscanf(fic_data,"%3d%8c%4c%2c%2c%c",&numstation,stationname,&stryear,&strmonth,&strday,&strhalf); 
	    firstdate.year=atoi(stryear);
	    firstdate.month=atoi(strmonth);
	    firstdate.day=atoi(strday);
	    if(atoi(strhalf)==1) firstdate.second=0.0;
	    else firstdate.second=43200.0;
	    rewind(fic_data);

	    nmes=read_hawaii(fic_data,&lat,&lon,datatime,sealevel,firstdate,conf.nbdata);

	    rewind(fic_data);
	    fscanf(fic_data,"%3d%8c%4d",&numstation,stationname,&firstdate.year);
	    fclose(fic_data);
	    stationname[8]='\0';
	    for(i=0;i<7;i++)if((isspace(stationname[i])) && !(isspace(stationname[i+1]))) stationname[i]='_';
	    i=0;
	    while((stationname[i] != ' ')&&(i<9)) i++;
	    stationname[i]='\0';

	    printf("\nStation: %s LAT=%.2lf LON=%.2lf\n",stationname,lat,lon);
	    printf("%d data; ",nmes);


	    Hmask=9999.0;
	    nmes=unmask(datatime,sealevel,nmes,Hmask) ;
	    printf("%d valid data\n",nmes);
	    break;
	  };
	  
	default: quit("unknown format");
	}/* end switch format */

/*       if (clean>0) */
/* 	{ */
/* 	  printf("%d-sigma filter on initial signal\n",clean); */
/* 	  nmes=Xsigma_filter (datatime,sealevel,nmes,Hmask,clean); */
/* 	} */

      alloc_copy(conf.nbdata,&sealevelcopy);
      alloc_copy(conf.nbdata,&ibcopy);
      alloc_copy(conf.nbdata,&eccopy);
      alloc_copy(conf.nbdata,&residual);
      alloc_copy(conf.nbdata,&S_tide);
           
      mean=gsl_stats_mean(sealevel,1,nmes);
      for (i=0;i<nmes;i++)  sealevelcopy[i]=sealevel[i]-mean;
      
      fprintf(conf.readme_file,"data file:\n%s\n",conf.data_filename);
      fprintf(conf.readme_file,"\narchive directory:\n%s\n",conf.directory_path);
      rms1=gsl_stats_sd(sealevelcopy,1,nmes);
      if (conf.format == 'h') fprintf(conf.readme_file,"\n\nRMS before analysis: %8.3f mm\n",rms1);
      else fprintf(conf.readme_file,"\n\nRMS before analysis: %8.3f [Z unit]\n",rms1);
	   
      for (i=0;i<34;i++) {tabZ[i].reel=0.;tabZ[i].imag=0.;}

      /*--------------------------------------------------------*/
      /*  polygone filtering            (optional)              */
      /*--------------------------------------------------------*/
      filtre=1;
      if (conf.filter_type==1) filtre=geo_filter(lat,lon,conf.geo,conf.filter_type);
      else if (conf.filter_type==2) filtre=bassin_filter(lat,lon,conf.poly_filename);


      if (filtre==1)
	{
	     
	  trecord=((datatime[nmes-1]-datatime[0])/24);

	  if (conf.nbond<nmes/2)
	    { 
	      cnt++;

	      /*------------------------------*/
	      /* spectrum checking            */
	      /*------------------------------*/
	      printf("\nChecking tidal spectrum ...\n");
	      fprintf(conf.readme_file,"\nSpectrum checking\n");

	      nA.h=old_nA;
	      switch (conf.analys_type) 
		{
		case (0): 
		  {checkspectrum(reference_s,&work_s,conf.Trepet,trecord,nopt,KEEP,conf.readme_file);break;}
		case (1): 
		  {check_ortho(reference_s,&work_s,conf.Trepet,trecord,nopt,KEEP,conf.readme_file);break;}
		case (2): 
		  {nA.h=check_mixte_option(mixteLIST,&mixtework, conf.Trepet,trecord,nopt,KEEP,nA);break;}
		default: quit("analysis type");
		}
	      trans=0;

	      for(i=0;i<reference_s.n;i++) 
		if (reference_s.wave[i].code==work_s.wave[i-trans].code) tup[i]=0;
		else
		  {
		    tup[i]=1;
		    trans++;
		  }


	      /*------------------------------*/
	      /* Solid tide                   */
	      /*------------------------------*/
	      if (conf.solid_effect==1)
		{	 
		  printf("\nComputing the solid tide prediction ...\n");

		  time1=time(NULL);
		  terrestre=1;
		  for(i=0;i<nmes;i++)
		    {
		      S_tide[i]=0;
		      for(k=0;k<work_s.n;k++)
			{
			  wave=work_s.wave[k];
			  tidal_potential(wave,lat,lon, &Ap,&Gp,terrestre);
			  Z.reel= Ap*cos(-Gp);
			  Z.imag=Ap*sin(-Gp);
			  S_tide[i]+=Tide_prediction(datatime[i],wave,Z,0,conf.CTO);
			}
		      sealevel[i]-=S_tide[i];
		    }
		  time2=time(NULL);
		  printf("%2.1f min\n",difftime(time2,time1)/60.0);
		}


	      /*-------------------------------------------*/
	      /*  inverse barometer correction (optional)  */
	      /*-------------------------------------------*/

	      if ((conf.inv_bar)||(ib_mean))
		{
		  if(ib_mean)ib_mean=0;
		  printf("\nComputing the IB correction ...\n");

		  time1=time(NULL);
		  read_header(376974.17, &grid, &sampling, &masque, 0);
		  buf1=(float *)malloc(grid.nx*grid.ny*sizeof(float));
		  buf2=(float *)malloc(grid.nx*grid.ny*sizeof(float));
		     
		  for(i=0;i<nmes;i++) 
		    {
		      read_header(datatime[i], &grid, &sampling , &masque, 0);
			 
		      getcnesdate(datatime[i],&actual);
		      hour=floor(actual.second/3600.);
		      if(hour<9)
			{
			  delta_t1=24+hour-9;
			  delta_t2=-hour+9;
			}
		      else
			{
			  delta_t1=hour-9;
			  delta_t2=24-hour+9;
			}
		      t1=datatime[i]-delta_t1;
		      t2=datatime[i]+delta_t2;
			 
		      rstatus=read_meteo(t1, grid.nx, grid.ny, buf1,sampling);
		      rstatus=read_meteo(t2, grid.nx, grid.ny, buf2,sampling);

		      sealevel[i]= sealevel[i]+map_getf_xyt(grid, buf1, buf2, t1, t2, masque, lon, lat,datatime[i])/100;/*pour utiliser IB  - serie en METRE*/
		      ibcopy[i]=map_getf_xyt(grid, buf1, buf2, t1, t2, masque, lon, lat, datatime[i])/100;
		    }
		  time2=time(NULL);
		  printf("%2.1f min\n",difftime(time2,time1)/60.0);
		}

		  
	      /*--------------------------------------*/
	      /*  loading correction (optional)       */
	      /*--------------------------------------*/

	      if (conf.load_effect)
		{
		  printf("\nComputing the loading correction ...\n");

		  time1=time(NULL);
		  for(i=1;i<34;i++) 
		    {
		      n=0;
		      rstatus=Wave_select(i,&wave,0);
		      
		      if (rstatus)
			{
			  sprintf(input,"%s/drfes99.%s.bimg",flpath,wave.name);
			  loadingEC(input,lat,lon,&Z,&mask);
			  if((Z.reel!=mask.reel)&&(Z.imag!=mask.imag))
			    while(n<nmes)
			      { 
				Echarge=Tide_prediction(datatime[n],wave,Z,0,conf.CTO); 
				sealevel[n]=sealevel[n]-Echarge; 
				eccopy[n]+=Echarge;
				n++;
			      }
			  else break;  /* exit !! */
			}
		    }
		  time2=time(NULL);
		  printf("%2.1f min\n",difftime(time2,time1)/60.0);
		}

	      rms1=gsl_stats_sd(sealevel,1,nmes);
	      if (conf.format == 'h') fprintf(conf.readme_file,"\n\nRMS after optional corections: %8.3f mm\n",rms1);
	      else fprintf(conf.readme_file,"\n\nRMS after optional corections: %8.3f [Z unit]\n",rms1);
	      
	 


	      switch(conf.analys_type)
		{
		case(0):
		  {
		    /*---------------------------------*/
		    /* iterative harmonic analysis     */
		    /*---------------------------------*/
		    time1=time(NULL);
		    printf("\nComputing harmonic analysis ...\n");
		    if(conf.CPU==1) /*parra*/
		      harmonic_analysis(sealevel,datatime,nmes, work_s, nopt,lon,lat,tabZ_sortie,residual,KEEP,conf.CTO);
		    else/*parra*/
		      harmonic_analysis_parra(sealevel,datatime,nmes, work_s, nopt,lon,lat,tabZ_sortie,residual,KEEP,conf.CTO,conf.CPU);
		    time2=time(NULL);
		    printf("%2.1f min\n",difftime(time2,time1)/60.0);
		    break;
		  }
		case(1):
		  {
		    /*---------------------------------*/
		    /* orthotide analysis              */
		    /*---------------------------------*/
		    printf("\nComputing orthotide analysis ...\n");
		    time1=time(NULL);
		    orthotide_analysis(sealevel,datatime,nmes,work_s,nopt,lon,lat,tabZ_sortie,conf.CTO);
		    time2=time(NULL);
		    printf("%2.1f min\n",difftime(time2,time1)/60.0);
		    break;
		  }

		case(2):
		  {
		    /*---------------------------------*/
		    /* mixte analysis                  */
		    /*---------------------------------*/
		    printf("\nComputing mixed analysis ...\n");
		    
		    /*     if (nA.o==0) nb_group=0;*/
		    time1=time(NULL);
		    mixte_analysis(nmes,sealevel,datatime, mixtework, tabZ_sortie,KEEP, nA ,nb_group,conf.CTO);
		    time2=time(NULL);
		    printf("%2.1f min\n",difftime(time2,time1)/60.0);
		    work_s.n=mixtework.n;
		    break;
		  }

		}/*end switch*/



	     
	      if ( (amplitude=calloc(work_s.n,sizeof(float))) == NULL) quit("can't allocate amplitude");
	      if ( (retard=calloc(work_s.n,sizeof(float))) == NULL) quit("can't allocate retard");
	      if ( (noise=calloc(work_s.n,sizeof(double))) == NULL) quit("can't allocate noise");
	      
	      /*------------------------------*/
	      /*  outputs editing             */
	      /*------------------------------*/
	      
	      if(conf.analys_type==0)
		{
		  rms2=gsl_stats_sd(residual,1,nmes);
		  if (conf.format == 'h') fprintf(conf.readme_file,"\n\n\nRMS after analysis: %8.3f mm ",rms2);
		  else fprintf(conf.readme_file,"\n\n\nRMS after analysis: %8.3f [Z unit] ",rms2);	
		  fprintf(conf.readme_file,"(%5.2f pct of sea level RMS)",rms2*100/rms1);
		}
	      else fprintf(conf.readme_file,"no residual computed\n");
	      
	      trans=0;
	      
	      for (i=0;i<work_s.n;i++) 
		{
		  amplitude[i]=module(tabZ_sortie[i]);
		  retard[i]=phase(tabZ_sortie[i]);
		} 
	      
	      
	   		
	      /*-------------------------------------------*/
	      
	      
	      
	      /* ---- .sum file editing ---- */
	      printf("\nWriting %s.sum summary file ...\n",conf.rootname);
		  
	      fprintf(conf.sum_file,"#-----------------------------------------------------\n");
	      fprintf(conf.sum_file,"Pt  : %d\n",nopt);
	      fprintf(conf.sum_file,"lon : %lf\n",lon);
	      fprintf(conf.sum_file,"lat : %lf\n",lat);
	      fprintf(conf.sum_file,"Mes : %d\n",nmes);
	      /* datatime in CNES days */ 
	      for(i=0;i<nmes;i++)  fprintf(conf.sum_file,"%13.6f  %9.4f  %9.4f  %9.4f  %9.4f  %9.4f\n",datatime[i]/24.0,sealevelcopy[i],S_tide[i],ibcopy[i],eccopy[i],residual[i]);
	      /*-------------------------------------------*/
	      
	      
	      /* ---- .dat residual file editing ---- */
	      if ((conf.Trepet == 0.041666) && (conf.format=='h'))
		{
		  printf("\nWriting %s.res.%d.dat residual serie file ...\n",conf.rootname,work_s.n);
		  
		  sprintf(input,"%s/%s.res.%d.dat",conf.directory_path,conf.rootname,work_s.n);
		  res_dat=fopen(input,"w");
		  save_hawaii(res_dat,lat,lon,residual,datatime,numstation,stationname,firstdate,nmes,datatime[nmes-1]);
		  fclose(res_dat);
		}
	      /*--------------------------------------*/
	      
	      
	      /* ---- .IOS.ARM file creating  ---- */
	      if(conf.ios_output==1)
		{
		  printf("\nWriting %s.ios.arm harmonic constants file ...\n",conf.rootname);
		  
		  if(conf.format=='l')
		    if (conf.Trepet==9.9156) sprintf(input,"%s/ondes_Pt%d.ios.arm",conf.directory_path,nopt);
		    else sprintf(input,"%s/%s.ios.arm",conf.directory_path,conf.rootname);
		  if(conf.format=='h') sprintf(input,"%s/%s.ios.arm",conf.directory_path,conf.rootname);
		  
		  conf.ios_file=fopen(input,"a+");
		  if(conf.Trepet==35) sprintf(stationname,"ERS_%d",nopt);
		  if(conf.Trepet==9.9156) sprintf(stationname,"TP_%d",nopt);
		  sortieIOS_std(conf.format,nopt,nmes,lat,lon,work_s,amplitude,retard,conf.ios_file,stationname,numstation,datatime[0],datatime[nmes-1],datatime[nmes/2],rms1,rms2,mean); 
		  fclose(conf.ios_file);
		}
	      /*-------------------------------------------*/


		  
	      /* ---- .MGR file creating  ---- */
	      if (conf.mgr_output==1)
		{
		  printf("\nWriting %s.mgr harmonic constants file ...\n",conf.rootname);

		  if(conf.Trepet==35) {sprintf(stationname,"ERS");numstation=nopt;}
		  if(conf.Trepet==9.9156) {sprintf(stationname,"TP");numstation=nopt;}
		  sprintf(input,"%s",conf.program_name);
		  getcnesdate(datatime[0],&actual);
		  startyear=actual.year;
		  getcnesdate(datatime[nmes-1],&actual);
		  endyear=actual.year;
		  
		  if(conf.analys_type==2) sortie_MGR(lat,lon,mixtework,amplitude,retard,conf.mgr_file,conf.format,conf.Trepet,numstation,stationname,startyear,endyear,trecord,input);
		  else sortie_MGR(lat,lon,work_s,amplitude,retard,conf.mgr_file,conf.format,conf.Trepet,numstation,stationname,startyear,endyear,trecord,input);
		}

	      /*-------------------------------------------*/
	      
	      
	      /*------ FFT of the residual signal ---------*/
	      if ( (conf.fft) && (conf.Trepet == 0.041666) )
		{
		  printf("\nComputing residual serie FFT...\n");
		  time1=time(NULL);
		  FFT_compute(residual,datatime,nmes,conf.Trepet,conf.directory_path,conf.rootname,work_s,noise);
		  time2=time(NULL);
		  printf("%3.1f min\n",difftime(time2,time1)/60.0);

		  /* error bars */
		  printf("\nWriting %s.bar.arm file ...\n",conf.rootname);
		  sprintf(input,"%s/%s.bar.arm",conf.directory_path,conf.rootname);
		  fft_file=fopen(input,"w");
		  sortie_BAR(conf.format,nopt,nmes,lat,lon,work_s,amplitude,retard,fft_file,stationname,numstation,datatime[0],datatime[nmes-1],datatime[nmes/2],rms1,rms2,mean,noise); 
		  fclose(fft_file);
		}

	      /*-------------------------------------------*/
	      
	      /*-------------------------------------------*/
	      if (conf.drift)
		{
		  printf("\nComputing the drift of the sea level serie ...\n");

		  if(conf.format=='h') fprintf(conf.readme_file,"\n\nsea level change: %+4.0lf mm in ~%3.1lf years\n\n",trend_compute(sealevelcopy,datatime,nmes),trecord/365.25);		  
		  else fprintf(conf.readme_file,"\n\nsea level change: %+4.0lf [Z unit] in ~%3.1lf years\n\n",trend_compute(sealevelcopy,datatime,nmes),trecord/365.25);		  
		}
	      /*-------------------------------------------*/
		  
	      free(amplitude);
	      free(retard);
	      free(noise);
		  
		  
	    }/*if nbond<nmes/2 */
	  else
	    { 
	      sprintf(input,"%s/PT_%d.error",conf.directory_path,nopt);
	      error=fopen(input,"w");
	      ERROR(nopt,nmes,conf.nbond,lat,lon,error);
	      flag_error=1;
	    }/*else nbond<nmes/2 */
	     
	}/* if filtre*/
	 
      if (filtre==1) 
	if (flag_error) printf("\nanalysis n=%d ERROR because of not enought measures\n",nopt);
	else printf("\n#########analysis n=%d completed#########\n",nopt);
      else 
	if(conf.nXpoints>20000) { if(prin_cnt>50){printf ("\nanalysis n=%d ERROR because of filtering\n",nopt);prin_cnt=0;}}
	else printf ("\nanalysis n=%d ERROR because of filtering\n",nopt);      
    }/*while nopt<nXpoints */
 
  printf("\n>>>>>>>>>> execution completed >>>>>>>>>>>>\n\n");



  if ((conf.inv_bar) || (ib_mean) ) 
    {
      free(buf1);
      free(buf2);
    }
  free(sealevel);
  free(sealevelcopy);
  free(datatime);
  free(KEEP);
  free(tabZ_sortie);
  free(work_s.wave);
  free (reference_s.wave); 
  if(conf.format=='l') fclose(fic_data);
  fclose(conf.mgr_file);
  fclose(conf.readme_file);
  free(ibcopy);
  free(eccopy);
  free(residual);
  free(S_tide);
  free(tup);
  free(conf.directory_path);
  if (conf.analys_type==2)free(position);
  if (conf.analys_type==2==2) free (mixteLIST.wave); 
}/*end.*/


