#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <math.h> 

#include "analyse_alti.h"
#include "aktarus.h"

/*------------------------------------------------------------------------
  
  Nom         :  bmg_readheader

--------------------------------------------------------------------------*/

int bmg_readheader (FILE *in,bmg_fheader *header)
  {
  
  int nlevel,i,status, size;
  size_t offset, nread; 
  float time;
 
  status=0;

  offset=4;

  nread=fread(&size, sizeof(int), (size_t) 1, in );
#ifdef SWAPIO
  size=long_swap(size);
#endif

  if(size !=80) 
    {
    status=-1;
    return(0);
    } 
  nread=fread(header->comment[0], (size_t) 80, (size_t) 1, in );
  fseek(in,offset,SEEK_CUR);  

  fseek(in,offset,SEEK_CUR);  
  nread=fread(header->comment[1], (size_t) 80, (size_t) 1, in );
  fseek(in,offset,SEEK_CUR);  

  fseek(in,offset,SEEK_CUR);  
  nread=fread(header->comment[2], (size_t) 80, (size_t) 1, in );
  fseek(in,offset,SEEK_CUR);  

  fseek(in,offset,SEEK_CUR);  
  nread=fread(header->comment[3], (size_t) 80, (size_t) 1, in );
  fseek(in,offset,SEEK_CUR);  

  nread=fread(&size, sizeof(int), (size_t) 1, in );
  nread=fread(&header->ni, sizeof(int), (size_t) 1, in );
  nread=fread(&header->nj, sizeof(int), (size_t) 1, in );
  nread=fread(&header->nk, sizeof(int), (size_t) 1, in );
  nread=fread(&header->nt, sizeof(int), (size_t) 1, in );
  nread=fread(&header->nd, sizeof(int), (size_t) 1, in );
/*   printf("header.ni=%d, header.nj=%d\n",header->ni,header->nj); */

  nlevel=header->nk;

#ifdef SWAPIO
  size=long_swap(size);
  nlevel=long_swap(nlevel);
#endif

  if(size == 24) 
    {
    nread=fread(&header->code, sizeof(int), (size_t) 1, in );
    header->size=4*(4+80+4)+(4+24+4)+(4+20+4)+(4+nlevel*4+4);
    }
  else
    {
    header->size=4*(4+80+4)+(4+20+4)+(4+20+4)+(4+nlevel*4+4);
    }
  fseek(in,offset,SEEK_CUR);  

  fseek(in,offset,SEEK_CUR);  
  nread=fread(&header->xmin, sizeof(float), (size_t) 1, in );
  nread=fread(&header->ymin, sizeof(float), (size_t) 1, in );
  nread=fread(&header->dx,   sizeof(float), (size_t) 1, in );
  nread=fread(&header->dy,   sizeof(float), (size_t) 1, in );
  nread=fread(&header->spec, sizeof(float), (size_t) 1, in );
  fseek(in,offset,SEEK_CUR);

  header->levels=(float *)malloc(nlevel*sizeof(float)); 

  fseek(in,offset,SEEK_CUR);  
  nread=fread(header->levels, sizeof(float), (size_t) nlevel, in );
  fseek(in,offset,SEEK_CUR);

  status=0;

#ifdef SWAPIO
/*  header=hearder_swap(header);*/
  header->ni	= long_swap(header->ni);
  header->nj	= long_swap(header->nj);
  header->nk	= long_swap(header->nk);
  header->nt	= long_swap(header->nt);
  header->nd	= long_swap(header->nd);
  header->code	= long_swap(header->code);
  header->xmin	= float_swap(header->xmin);
  header->ymin	= float_swap(header->ymin);
  header->dx	= float_swap(header->dx);
  header->dy	= float_swap(header->dy);
  header->spec	= float_swap(header->spec);
  for(i=0;i<nlevel;i++)
    header->levels[i]	= float_swap(header->levels[i]);
#endif

/*   bmg_printheader (*header); */

  return(status);

  }



/*------------------------------------------------------------------------
  
  Nom     : bmg_saver1


  Comments: bmg_saver1 save an unique time frame,dimension,level

--------------------------------------------------------------------------*/

int bmg_saver1 (char* file, int k, int d, int t, grid2_t grid, float *buf)
  {
  
  FILE *out;
  int i,option,status,dmax,kmax,tmax;
  size_t offset, nwrite, size, buf_size;
  size_t tsize,dsize,ksize,shift;
  bmg_fheader header;

  out=fopen(file,"rb+wb");
  
  offset=4;

  bmg_readheader (out, &header);

  kmax=header.nk;
  dmax=header.nd;
  tmax=header.nt;

  ksize=header.ni*header.nj*sizeof(float)+2*offset;
  dsize=header.nk*ksize;
  tsize=header.nt*dsize+sizeof(float)+2*offset;

  shift=(t-1)*tsize;
  fseek(out,shift,SEEK_CUR);  
  
  buf_size=header.ni*header.nj;

  shift=(d-1)*dsize+(k-1)*ksize+3*offset;
  fseek(out,shift,SEEK_CUR);

#ifdef SWAPIO
  for (i=0;i<buf_size;i++) buf[i]=float_swap(buf[i]);
#endif

  size=header.ni*header.nj*sizeof(float);
#ifdef SWAPIO
  size=long_swap(size);
#endif
  nwrite=fwrite(&size, sizeof(int), (size_t) 1, out );
  nwrite=fwrite(buf,sizeof(float),(size_t)header.ni*header.nj,out);
  nwrite=fwrite(&size, sizeof(int), (size_t) 1, out );

#ifdef SWAPIO
    for (i=0;i<buf_size;i++) buf[i]=float_swap(buf[i]);
#endif

    free(header.levels);
  fclose(out);
  return(0);
  }



/*------------------------------------------------------------------------
  
  Nom         :  bmg_writeheader

--------------------------------------------------------------------------*/

int bmg_writeheader (FILE *in,bmg_fheader header)
  {
  
  int nlevel,i,status;
  size_t offset, nwrite, nitems;
  int    size;
  float time;
 
  status=0;

  offset=4;
  
  size=80;
#ifdef SWAPIO
  size=long_swap(size);
#endif

  nwrite=fwrite(&size, sizeof(int), (size_t) 1, in );
  if(nwrite !=1) 
    {
    status=-1;
    return(status);
    }
  nwrite=fwrite(header.comment[0], (size_t) 80, (size_t) 1, in );
  nwrite=fwrite(&size, sizeof(int), (size_t) 1, in );

  nwrite=fwrite(&size, sizeof(int), (size_t) 1, in );
  nwrite=fwrite(header.comment[1], (size_t) 80, (size_t) 1, in );
  nwrite=fwrite(&size, sizeof(int), (size_t) 1, in );

  nwrite=fwrite(&size, sizeof(int), (size_t) 1, in );
  nwrite=fwrite(header.comment[2], (size_t) 80, (size_t) 1, in );
  nwrite=fwrite(&size, sizeof(int), (size_t) 1, in );

  nwrite=fwrite(&size, sizeof(int), (size_t) 1, in );
  nwrite=fwrite(header.comment[3], (size_t) 80, (size_t) 1, in );
  nwrite=fwrite(&size, sizeof(int), (size_t) 1, in );

  size=6*sizeof(int);
#ifdef SWAPIO
  size=long_swap(size);
#endif
  nwrite=fwrite(&size, sizeof(int), (size_t) 1, in );
  nwrite=fwrite(&header.ni, sizeof(int), (size_t) 1, in );
  nwrite=fwrite(&header.nj, sizeof(int), (size_t) 1, in );
  nwrite=fwrite(&header.nk, sizeof(int), (size_t) 1, in );
  nwrite=fwrite(&header.nt, sizeof(int), (size_t) 1, in );
  nwrite=fwrite(&header.nd, sizeof(int), (size_t) 1, in );
  nwrite=fwrite(&header.code, sizeof(int), (size_t) 1, in );
  nwrite=fwrite(&size, sizeof(int), (size_t) 1, in );
/*   printf("header.ni=%d, header.nj=%d\n",header.ni,header.nj); */

  size=5*sizeof(float);
#ifdef SWAPIO
  size=long_swap(size);
#endif
  nwrite=fwrite(&size, sizeof(int), (size_t) 1, in );
  nwrite=fwrite(&header.xmin, sizeof(float), (size_t) 1, in );
  nwrite=fwrite(&header.ymin, sizeof(float), (size_t) 1, in );
  nwrite=fwrite(&header.dx,   sizeof(float), (size_t) 1, in );
  nwrite=fwrite(&header.dy,   sizeof(float), (size_t) 1, in );
  nwrite=fwrite(&header.spec, sizeof(float), (size_t) 1, in );
  nwrite=fwrite(&size, sizeof(int), (size_t) 1, in );

  size=header.nk*sizeof(float);
#ifdef SWAPIO
  size=long_swap(size);
#endif
  nwrite=fwrite(&size, sizeof(int), (size_t) 1, in );
  nitems=header.nk;
  nwrite=fwrite(header.levels, sizeof(float), nitems, in );
  nwrite=fwrite(&size, sizeof(int), (size_t) 1, in );

  status=0;

  return(status);

  }



/*------------------------------------------------------------------------
  
  Nom         :  bmg_createheader

--------------------------------------------------------------------------*/

int bmg_createheader_r (char* file,int kmax,int dmax,int tmax,grid2_t grid,float *levels,float mask)
  {  
  FILE *out;
  float time;
  int i,option,status,d,t,k,cnt;
  size_t offset, nwrite, size;
  size_t tsize,dsize,ksize,shift;
  bmg_fheader header;

  out=fopen(file,"wb");
  
  offset=4;

  strcpy(header.comment[0],"no comment");
  strcpy(header.comment[1],"no comment");
  strcpy(header.comment[2],"no comment");
  strcpy(header.comment[3],"no comment");
  header.ni=grid.nx;
  header.nj=grid.ny;
  header.xmin=grid.xmin;
  header.ymin=grid.ymin;
  header.dx=grid.dx;
  header.dy=grid.dy;
  header.nk=kmax;
  header.nd=dmax;
  header.nt=tmax;
  header.spec=mask;
  header.levels=levels;

  bmg_writeheader (out, header);

  fclose(out);
  return(0);

  }


/*------------------------------------------------------------------------*/
void bmg_loadc1_analyse_alti (char* filename, int k,int d,int t,
                  grid_t *grid, int *n, fcomplex **buffer, fcomplex *mask, int *status)
/*------------------------------------------------------------------------*/
  {
   
  FILE *in;
  int option,i,lstatus;
  size_t offset, nread, size;
  float time,*dum;
  bmg_fheader header;

  if(!(in=fopen(filename,"rb")))
    {
      *status=-1;
      printf("can not open file %s\n",filename);
      return;
    }


     *status=0; 
  
  offset=4;

  *status=bmg_readheader(in,&header);
  if(*status != 0) goto error;

  grid->xmin=header.xmin;
  grid->ymin=header.ymin;

  grid->dx=header.dx;
  grid->dy=header.dy;

  grid->nx=header.ni;
  grid->ny=header.nj;

  grid->xmax=grid->xmin+(grid->nx-1)*grid->dx;
  grid->ymax=grid->ymin+(grid->ny-1)*grid->dy;

  grid->Mode=0;
/*   free(header.levels); */
  
  /* pas de pointer d'axes*/

  *n =  header.ni;

  lstatus=fseek(in,offset,SEEK_CUR);  
  if(lstatus != 0) goto error;
  nread=fread(&time, sizeof(float), (size_t) 1, in );
  if(nread != 1) goto error;
  if(lstatus != 0) goto error;
  lstatus=fseek(in,offset,SEEK_CUR);

#ifdef SWAPIO
  time= float_swap(time);
#endif

  *buffer=(fcomplex *) malloc(grid->nx*grid->ny*sizeof(fcomplex));

  if(*buffer==NULL) goto error;

  lstatus=fseek(in,offset,SEEK_CUR);  
  if(lstatus != 0) goto error;
  nread=fread(*buffer, sizeof(fcomplex), (size_t) header.ni* header.nj, in );
  if(nread != header.ni* header.nj) goto error;
  lstatus=fseek(in,offset,SEEK_CUR);
  if(lstatus != 0) goto error;

#ifdef SWAPIO
  for (i=0;i<header.ni* header.nj;i++) 
    (*buffer)[i]=complex_swap((*buffer)[i]);
#endif

  fclose(in);

  mask->reel= header.spec;
  mask->imag= header.spec;
  return;

  error:
  *status=-1;
  return;

  }


/*------------------------------------------------------------------------
  
  Nom         :  bmg_savec1

--------------------------------------------------------------------------*/

int bmg_savec1 (char* filename,int kmax,grid_t grid,int n,fcomplex *buf,float *levels,fcomplex mask)
  {
  
  FILE *out;
  float time;
  int i,option,status,d,t,k,size;
  size_t offset, nwrite;
  size_t tsize,dsize,ksize,shift;
  bmg_fheader header;

  printf("#bmg_savec1 ... starting\n");

  out=fopen(filename,"wb");
  
  offset=4;

  strcpy(header.comment[0],"no comment");
  strcpy(header.comment[1],"no comment");
  strcpy(header.comment[2],"no comment");
  strcpy(header.comment[3],"no comment");
  header.ni=grid.nx;
  header.nj=grid.ny;
  header.xmin=grid.xmin;
  header.ymin=grid.ymin;
  header.dx=grid.dx;
  header.dy=grid.dy;
  header.nk=kmax;
  header.nd=1;
  header.nt=1;
  header.spec=mask.reel;
  header.levels=levels;

  bmg_writeheader (out, header);

  ksize=header.ni*header.nj*sizeof(fcomplex)+2*offset;
  dsize=header.nk*ksize;
  tsize=header.nt*dsize+sizeof(float)+2*offset;
  
#ifdef SWAPIO
  for (i=0;i<header.ni* header.nj*kmax;i++) buf[i]=complex_swap(buf[i]);
#endif

  d=1;
  t=1;

#ifdef SWAPIO
  time= float_swap(time);
#endif

  size=sizeof(float);
#ifdef SWAPIO
  size=long_swap(size);
#endif

  nwrite=fwrite(&size, sizeof(int), (size_t) 1, out );
  nwrite=fwrite(&time, sizeof(float), (size_t) 1, out );
  nwrite=fwrite(&size, sizeof(int), (size_t) 1, out );

  size=header.ni*header.nj*sizeof(fcomplex);
#ifdef SWAPIO
  size=long_swap(size);
#endif
  for(k=0;k<kmax;k++)
    {
    nwrite=fwrite(&size, sizeof(int), (size_t) 1, out );
    nwrite=fwrite(&buf[k*header.ni*header.nj], sizeof(fcomplex), (size_t) header.ni* header.nj, out );
    nwrite=fwrite(&size, sizeof(int), (size_t) 1, out );
    }

#ifdef SWAPIO
    for (i=0;i<header.ni* header.nj*kmax;i++) buf[i]=complex_swap(buf[i]);
#endif

  fclose(out);
  printf("#bmg_savec1 ... finished\n");

  }

