#include <stdio.h> 
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <strings.h>
#include <limits.h>
#include <malloc.h>  
#include <alloca.h>

#include "aktarus.h"

#include "gsl/gsl_fft_real.h"
#include "gsl/gsl_errno.h"
#include "gsl/gsl_statistics_double.h"
#include "gsl/gsl_sort.h"
#include "gsl/gsl_math.h"


/*-------------------------------------------------------------------------------*/
float module(fcomplex Z)
/*-------------------------------------------------------------------------------*/
{
  float mod;
  mod=sqrt(Z.reel*Z.reel+Z.imag*Z.imag);
  return(mod);
}

/*-------------------------------------------------------------------------------*/
float phase(fcomplex Z)
/*-------------------------------------------------------------------------------*/
{
  float phi;
  phi=360.0*atan2(Z.imag,Z.reel)/(2*pi);
  if(phi < 0.) phi+=360.;
  return(phi);
}



/*----------------------------------------------------------------------------*/
int map_interpolationc(grid_t grid,int n, fcomplex *buf, fcomplex mask, float x, float y, fcomplex *z)
/*----------------------------------------------------------------------------*/
{
  int k,l,k0,l0;
  float s,w,xx,yy;
  fcomplex r;

  if(x < grid.xmin)
    {
    *z=mask;
    printf("#Error in map_interpolation: xmin= %f and x= %f \n", grid.xmin,x);
    return (-1);
    }
    
  if (x > grid.xmax)
    {
    *z=mask;
    printf("#Error in map_interpolation: xmax= %f and x= %f \n", grid.xmax,x);
    return (-1);
    }
 
  if(y < grid.ymin)
    {
    *z=mask;
    printf("#Error in map_interpolation: ymin= %f and y= %f \n", grid.ymin,y);
    return (-1);
    }
 
  if(y > grid.ymax)
    {
    *z=mask;
    printf("#Error in map_interpolationc: ymax= %f and y= %f \n", grid.ymax,y);
    return (-1);
    }

  k0= (int) ((x-grid.xmin)/grid.dx);
  l0= (int) ((y-grid.ymin)/grid.dy);
 
  z->reel=0.0;
  z->imag=0.0;
  w=0.0;

  for( k=k0; k <= k0+1; k++)
  {
    xx=grid.xmin+k*grid.dx;
    if (k < 0)       break;
    if (k > grid.nx-1) break;
    for( l=l0; l <= l0+1; l++)
    {
      yy=grid.ymin+l*grid.dy;
      if(l < 0)       break;
      if(l > grid.ny-1) break;
      s=(grid.dx-fabs(x-xx))*(grid.dy-fabs(y-yy));
      r=buf[k+n*l];
      if ((r.reel != mask.reel) && (r.imag != mask.imag))
      {
        z->reel=z->reel+r.reel*s;
        z->imag=z->imag+r.imag*s;
        w=w+s;
      }
    }
  }

if (w > 0.0) 
  {
  z->reel /=w;
  z->imag /=w;
  }
else         *z=mask;

return(0);
}
/*----------------------------------------------------------------------------*/
int force_interpolationc(grid_t grid,int n, fcomplex *buf, fcomplex mask, float x, float y, fcomplex *z)
/*----------------------------------------------------------------------------*/
{
  int k,l,k0,l0;
  float s,h,ref,xx,yy;
  fcomplex r;

  if(x < grid.xmin)
    {
    *z=mask;
    printf("#Error in force_interpolation: xmin= %f and x= %f \n", grid.xmin,x);
    return (-1);
    }
    
  if (x > grid.xmax)
    {
    *z=mask;
    printf("#Error in force_interpolation: xmax= %f and x= %f \n", grid.xmax,x);
    return (-1);
    }
 
  if(y < grid.ymin)
    {
    *z=mask;
    printf("#Error in force_interpolation: ymin= %f and y= %f \n", grid.ymin,y);
    return (-1);
    }
 
  if(y > grid.ymax)
    {
    *z=mask;
    printf("#Error in force_interpolationc: ymax= %f and y= %f \n", grid.ymax,y);
    return (-1);
    }

  k0= (int) ((x-grid.xmin)/grid.dx);
  l0= (int) ((y-grid.ymin)/grid.dy);
 
  z->reel=0.0;
  z->imag=0.0;
  h=0.0;
  ref=FLT_MAX;

  for( k=k0-1; k <= k0+2; k++)
  {
    xx=grid.xmin+k*grid.dx;
    if (k < 0)       break;
    if (k > grid.nx-1) break;
    for( l=l0-1; l <= l0+2; l++)
    {
      yy=grid.ymin+l*grid.dy;
      if(l < 0)       break;
      if(l > grid.ny-1) break;
      h=gsl_hypot(grid.dx-fabs(x-xx),grid.dy-fabs(y-yy));
      r=buf[k+n*l];
      if((h<ref)&&(r.reel!=mask.reel)) 
	{
	  z->reel=r.reel;
	  z->imag=r.imag;
	  ref=GSL_MIN(h,ref);
	}
    }
  }


return(0);
}

/*----------------------------------------------------------------------------*/
void found_z(int na,grid_t grid,double lat,double lon,fcomplex *buffer,fcomplex mask,fcomplex *z)
/*----------------------------------------------------------------------------*/
{
  int status;
  float x,y;
  x=(float) lon;
  y=(float) lat;

  if(x < grid.xmin) x+=360.;
  if(x > grid.xmax) x-=360.;
  status=map_interpolationc(grid,na,buffer,mask,x,y,z); 

  if( (z->reel==mask.reel) && (z->imag==mask.imag) ) /* addition LR, 08/09/2003 */
    status=force_interpolationc(grid,grid.nx,buffer,mask,x,y,z);


}
  
/*--------------------------------------------------------------------------------*/

int interpolate_tz(double *h, double *t, double mask, int n, double time, double *z)

/*--------------------------------------------------------------------------------*/

{
  int k,k1,k2;
  double r;

  if(time < t[0]) 
    {
      *z=mask;
      return(0);
    }

  if(time > t[n-1]) 
    {
      *z=mask;
      return(0);
    }

  k1=0;
  k2=n-1;

  while(k2-k1>1)
    {
      k=0.5*(k2+k1);
      if(time > t[k])
	{
	  k1=k;
	  continue;
	}
      if(time < t[k])
	{
	  k2=k;
	  continue;
	}
      if(time = t[k])
	{
	  *z=h[k];
	  return(0);
	}
    }

  if(h[k1]==mask)
    {
      *z=mask;
      return(0);
    }

  if(h[k2]==mask)
    {
      *z=mask;
      return(0);
    }

  r=(t[k2]-time)/(t[k2]-t[k1]);

  *z=r*h[k1]+(1.-r)*h[k2];

  return(1);

}


/*-------------------------------------------------------------------------------------------------*/

int resample(double sampling,double *h, double *t, int nmes,double mask)

/*-------------------------------------------------------------------------------------------------*/
{
  int i,n,status;
  double *h_resample,*t_resample;
  double r,t0;


  h_resample=(double *)malloc(nmes*sizeof(double));
  t_resample=(double *)malloc(nmes*sizeof(double));

  r=modf(t[0],&t0);
  i=0;
  do  i++;
  while (t0+i*sampling<=t[nmes-1]);
  n=i;

  for(i=0;i<n;i++) t_resample[i]=t0+i*sampling;

  for(i=0;i<n;i++)
    status=interpolate_tz(h,t,mask,nmes,t_resample[i],&(h_resample[i]));

  for(i=0;i<n;i++)
    {
      h[i]=h_resample[i];
      t[i]=t_resample[i];
    }

  free(h_resample);
  free(t_resample);

  return(n);

}



/*-------------------------------------------------------------------------------------------------*/

int unmask (double *t,double *sealevel,int nmes,double mask) 

/*-------------------------------------------------------------------------------------------------*/
{ 
  int i,n,j;
  double *level,*timing;

  level=(double *)malloc(nmes*sizeof(double));
  timing=(double *)malloc(nmes*sizeof(double));

  j=0;
  n=0;
  for(i=0;i<nmes;i++)
    if(sealevel[i] != mask)
      {
	level[j]=sealevel[i];
	timing[j]=t[i];
	j++;
	n++;
      }

 for(i=0;i<n;i++)
   {
     sealevel[i]=level[i];
     t[i]=timing[i];
   }

 free(level);
 free(timing);
 return(n);
}

/*-------------------------------------------------------------------------------------------------*/

int Xsigma_filter (double *t,double *sealevel,int nmes,double mask,int threshold) 

/*-------------------------------------------------------------------------------------------------*/
{ 
  int k,n,j,*keep;
  double *z;
  double mean,rms;

  z=malloc(nmes*sizeof(double));
  keep=malloc(nmes*sizeof(int));

  nmes=unmask(t,sealevel,nmes,mask);

  mean=gsl_stats_mean(sealevel,1,nmes);
  rms=gsl_stats_sd_m(sealevel,1,nmes,mean);

  for(k=0;k<nmes;k++) keep[k]=0;

  for(k=0,n=0;k<nmes;k++) 
    {
      z[k]=(sealevel[k]-mean);
      if(z[k]*z[k] > threshold*rms ) {keep[k]=1;n++;}
    }

  for(j=0,n=0;j<nmes;j++)
    {
      if (keep[j]==1) continue;
      else
	{
	  sealevel[n]=sealevel[j];
	  t[n]=t[j];
	  n++;
	}
    }

  printf("%d data rejected by filtering\n",nmes-n);
 return(n);
}






/*---------------------------------------------------------------*/

double trend_compute(double *sealevelcopy, double *time, int nmes)

     /*---------------------------------------------------------------*/
{
  double xmean,ymean,varx, covarxy;
  double slope;

  xmean=gsl_stats_mean(sealevelcopy,1,nmes);
  varx=gsl_stats_variance(sealevelcopy,1,nmes);
  covarxy=gsl_stats_covariance(sealevelcopy,1,time,1,nmes);
  slope=covarxy/varx;

  return(slope);
}



/*---------------------------------------------------------------*/

void chi_square_limits(int nu,double *chi_inf,double *chi_sup)
     
     /*---------------------------------------------------------------*/

/* cumulative Chi_square distribution from 'Data Analysis Methods in Physical Oceanography' */
/* except for nu=1,40...100 from Abramowitz 's handbook' */
/* nu is the degree of freedom of the distribution */
/* chi_inf -> inf. limit coef. for a 95% interval (ie 0.975 column) */
/* chi_sup -> sup. limit coef. for a 95% interval (ie 0.025 column) */

{
  double chi_inf1,chi_sup1,chi_inf2,chi_sup2;
  
  switch(nu)
    {
    case(1): {*chi_inf=5.024;*chi_sup=0.0000982;break;}
    case(2): {*chi_inf=7.38;*chi_sup=0.0506;break;}
    case(3): {*chi_inf=9.35;*chi_sup=0.216;break;}
    case(4): {*chi_inf=11.1;*chi_sup=0.484;break;}
    case(5): {*chi_inf=12.8;*chi_sup=0.831;break;}

    case(6): {*chi_inf=14.4;*chi_sup=1.24;break;}
    case(7): {*chi_inf=16.0;*chi_sup=1.69;break;}
    case(8): {*chi_inf=17.5;*chi_sup=2.18;break;}
    case(9): {*chi_inf=19.0;*chi_sup=2.70;break;}
    case(10): {*chi_inf=20.5;*chi_sup=3.25;break;}

    case(11): {*chi_inf=21.9;*chi_sup=3.82;break;}
    case(12): {*chi_inf=23.3;*chi_sup=4.40;break;}
    case(13): {*chi_inf=24.7;*chi_sup=5.01;break;}
    case(14): {*chi_inf=26.1;*chi_sup=5.63;break;}
    case(15): {*chi_inf=27.5;*chi_sup=6.26;break;}

    case(16): {*chi_inf=28.8;*chi_sup=6.91;break;}
    case(17): {*chi_inf=30.2;*chi_sup=7.56;break;}
    case(18): {*chi_inf=31.5;*chi_sup=8.23;break;}
    case(19): {*chi_inf=32.9;*chi_sup=8.91;break;}
    case(20): {*chi_inf=34.2;*chi_sup=9.59;break;}

    case(21): {*chi_inf=35.5;*chi_sup=10.3;break;}
    case(22): {*chi_inf=36.8;*chi_sup=11.0;break;}
    case(23): {*chi_inf=38.1;*chi_sup=11.7;break;}
    case(24): {*chi_inf=39.4;*chi_sup=12.4;break;}
    case(25): {*chi_inf=40.6;*chi_sup=13.1;break;}

    case(26): {*chi_inf=41.9;*chi_sup=13.8;break;}
    case(27): {*chi_inf=43.2;*chi_sup=14.6;break;}
    case(28): {*chi_inf=44.5;*chi_sup=15.3;break;}
    case(29): {*chi_inf=45.7;*chi_sup=16.0;break;}
    case(30): {*chi_inf=47.0;*chi_sup=16.8;break;}

    case(40): {*chi_inf=59.3417;*chi_sup=24.4331;break;}
    case(50): {*chi_inf=71.4202;*chi_sup=32.3574;break;}
    case(60): {*chi_inf=83.2976;*chi_sup=40.4817;break;}
    case(70): {*chi_inf=95.0231;*chi_sup=48.7576;break;}
    case(80): {*chi_inf=106.629;*chi_sup=57.1532;break;}
    case(90): {*chi_inf=118.136;*chi_sup=65.6466;break;}
    case(100): {*chi_inf=129.561;*chi_sup=74.2219;break;}

    default: {
      if(nu>=100) {*chi_inf=999.9;*chi_sup=999.9;break;}
      else 
	{
	  nu/=10;
	  nu*=10;
	  chi_square_limits(nu,&chi_inf1,&chi_sup1);
	  nu+=10;
	  chi_square_limits(nu,&chi_inf2,&chi_sup2);
	  *chi_inf=chi_inf1+(chi_inf2-chi_inf1)/10.0;
	  *chi_sup=chi_sup1+(chi_sup2-chi_sup1)/10.0;
	  break;
	}
    }
    }
}


/*---------------------------------------------------------------------------*/

void FFT_compute(double *residual, double *time, int nmes, double Trepet, char *rpath, char *stationname,wavelist work_s,double *noiselevel) 

     /*---------------------------------------------------------------------------*/
{
  int    i,j,k,delay,*gap,n,nprim,status,df;

  double *res,*ampli,*frequency,noise,fe,l_c,trecord,trend;
  double *res_part1,*res_part2;
  double mean,*mean_fi,*rms_fi,*a,*ai,*f;
  double chi_inf,chi_sup;

  FILE   *out;

  char   outfilename[256],dum[256];

  gsl_fft_real_wavetable * real_wavetable;
  gsl_fft_real_workspace * work;

  if ( (gap=(int *)malloc(nmes*sizeof(int)) ) == NULL ) quit("can't allocate gap array in FFT");

 
  /*---------------------------------------------------------------------------*/
  /* mean and trend substracting                                               */
  /*---------------------------------------------------------------------------*/

  mean=gsl_stats_mean(residual,1,nmes);
  /*   trend=trend_compute(residual,time,nmes); */
  for(i=0;i<nmes;i++) residual[i]=residual[i]-mean/* -trend */;


  /*---------------------------------------------------------------------------*/
  /* replacing blanks in time data by zeros                                    */
  /*---------------------------------------------------------------------------*/
  
  n=nmes;
  for (i=1;i<nmes;i++)
    {
      gap[i]=(int)(time[i]-time[i-1]); 
      if (gap[i]!=1) n+=gap[i];
    }
  
  if ( ( res=(double *)calloc(n,sizeof(double)) ) == NULL ) quit("can't allocate res pointer");
  
  delay=0;
  i=1;
  res[0]=residual[0];
     
  for(j=1;j<nmes;j++) 
    {
      if(gap[i]==1) res[j+delay]=residual[i];
      else
	{
	  for(k=j;k<(j+gap[i]);k++) res[k+delay]=0.0;
	  res[k+delay]=residual[i];
	  delay+=gap[i];
	}
      i++;
    }   
  
  free(gap);
    
  if(fmod(n,2.0)!=0) n--;
  /*---------------------------------------------------------------------------*/


  /*---------------------------------------*/
  /* Academic mono frequency signal (test) */
  /*---------------------------------------*/
  /*       n=10000; */
  /*       Trepet=0.45; */
  /*       for(i=0;i<n;i++) time[i]=Trepet*i; */
  /*       for(i=0;i<n;i++) res[i]=10*sin(pi2*0.7*time[i]+1); */
  /*        for (i=0;i<n;i++) fprintf(ini,"%5d %7.3lf %7.3f %1.0f %7.3f\n",i,time[i],res[i]); */
  /*       fclose(ini); */

  /*---------------------------------------------------------------------------*/
  /* FFT by gsl_fft_real_transform of the detided signal                       */
  /*---------------------------------------------------------------------------*/

  if(nmes<120000) status=1; /* approximative! */
  else status=0;     /* after such value for nmes, the gsl fft routine is not optimized*/

  switch(status)
    {
      /*------------------------------------*/
    case(1):/*ie record length < 15 years   */
      /*------------------------------------*/
      {
	if( (work=gsl_fft_real_workspace_alloc(n)) == NULL) quit("can't allocate FFT workspace");
	if( (real_wavetable=gsl_fft_real_wavetable_alloc(n)) == NULL) quit("can't allocate FFT wavetable");

	if( (frequency=(double *)calloc(n,sizeof(double)) ) == NULL) quit("can't allocate frequency pointer");
	if( (ampli=(double *)calloc(n,sizeof(double)) ) == NULL) quit("can't allocate amplitude pointer");

	status= gsl_fft_real_transform(res,1,n,real_wavetable,work ); 
	gsl_fft_real_wavetable_free(real_wavetable);
	gsl_fft_real_workspace_free(work);

	/*----------------------------------------*/
	/*             Normalisation              */
	/*----------------------------------------*/
	/* f(i) = (1/Nobs*Tech)*i                 */
	/* A(i) = (2/Nobs)*Tech*sqrt(Re*Re+Im*Im) */
	/* Nobs = Nb of data in the serie         */
	/* Tech = sampling period                 */
	/* Re, Im = real, imaginary part of FFT   */
	/*----------------------------------------*/
	fe=1.0/Trepet;
	for (i=1;i<n;i+=2)
	  {
	    frequency[i]=fe/(2.0*n)*i; /*  freq in 1/day   */
	    ampli[i]=(2.0*NINT(Trepet*24)/n)*sqrt(res[i]*res[i]+res[i+1]*res[i+1]); /* Trepet expressed in hour*/
	  }

	free(res);

	/* 	sprintf(outfilename,"%s/%s.fft",rpath,stationname); */
	/* 	out=fopen(outfilename,"w"); */
	/* 	for (i=1;i<n;i+=2) fprintf(out,"%9.5f %9.5lf\n",frequency[i],ampli[i]); */
	/* 	fclose(out); */


	/*---------------------------------------------------------------------------*/
	/* Averaging for freq-df/2 < freq <freq+df/2                                 */
	/*---------------------------------------------------------------------------*/

	if( (a=(double *)malloc(n/2*sizeof(double)) ) == NULL ) quit("can't allocate a pointer");
	if( (f=(double *)malloc(n/2*sizeof(double)) ) == NULL ) quit("can't allocate f pointer");
      
	for(i=1,j=0;i<n;i+=2,j++) a[j]=ampli[i];
	for(i=1,j=0;i<n;i+=2,j++) f[j]=frequency[i];
   	free(ampli);
   	free(frequency);

	trecord=(time[nmes-1]-time[0])/(24*365.25); /* in year */
	df=1+floor(trecord)/0.5;               /* df frequencies ( equiv. 0.5 years) average (!!!????!!) */

	if( (ai=(double *)calloc(df,sizeof(double)) ) == NULL ) quit("can't allocate ai\n");
	if( (mean_fi=(double *)calloc((n),sizeof(double)) ) == NULL ) quit("can't allocate mean_fi\n");
	if( (rms_fi=(double *)calloc((n),sizeof(double)) ) == NULL ) quit("can't allocate rms_fi\n");
      
	for(i=0;i<(n/2-df);i++)
	  {
	    for(j=i;j<i+df;j++) ai[j-i]=a[j];
	    mean_fi[i]=gsl_stats_mean(ai,1,df);
	    rms_fi[i]=gsl_stats_sd(ai,1,df);
	  }
	free(ai);

	sprintf(outfilename,"%s/%s.smoothed_fft",rpath,stationname);
	out=fopen(outfilename,"w");
	fprintf(out,"# --freq ---- ampl - mean over df - rms over df\n");
	for(i=0;i<=(n/2-df);i++) /*error: (n-df)/2 before */
	  fprintf(out,"%9.5f %9.5f %9.5f %9.5f\n",f[i+df/2],a[i+df/2],mean_fi[i],rms_fi[i]);

	/*-----------------------------------------------------------------------------------*/
	/* Noise level for the analysis of the wave k = mean PSD in the vincinity of f(wave) */
	/*-----------------------------------------------------------------------------------*/

	for(k=0;k<work_s.n;k++) 
	  for(i=0;i<(n/2-df);i++)
	    if( fabs(work_s.wave[k].omega*24.0/360.0-f[i+df/2]) < (fe/n) ) /* freq in 1/day */
	      {
		/* linear interpolation between the frequency before and after f(wave)*/
		noiselevel[k]=(mean_fi[i+1]-mean_fi[i])/(fe/n)*(work_s.wave[k].omega*24.0/360.0-f[i+df/2])+mean_fi[i];
		break;
	      }

	free(f);
	free(a);
	free(mean_fi);
	free(rms_fi);
	break;

      }/*end case 'short serie' */

      /*------------------------------------*/
    case(0):/*ie record length > 15 years   */
      /*------------------------------------*/
      {
	nprim=n/2;
	if(fmod(nprim,2.0)!=0) nprim--;

	if ( ( res_part1=(double *)calloc(nprim,sizeof(double)) ) == NULL ) quit("can't allocate res_part1");
	if ( ( res_part2=(double *)calloc(nprim,sizeof(double)) ) == NULL ) quit("can't allocate res_part2");

	if( (work=gsl_fft_real_workspace_alloc(nprim)) == NULL) quit("can't allocate FFT workspace");
	if( (real_wavetable=gsl_fft_real_wavetable_alloc(nprim)) == NULL) quit("can't allocate FFT wavetable");

	if( (frequency=(double *)calloc(nprim,sizeof(double)) ) == NULL) quit("can't allocate frequency");
  	if( (ampli=(double *)calloc(nprim,sizeof(double)) ) == NULL) quit("can't allocate ampli");

	/* 2 half series hourly sampled*/
	for(i=0;i<nprim;i++) res_part1[i]=res[i];
	for(i=nprim,j=0;j<nprim;i++,j++) res_part2[j]=res[i];
       
	/* 2 FFTs of the 2 half-series */
	/* not strictly equivalent to 1 FFT of the total serie (particularly in low frequencies)  */
	status= gsl_fft_real_transform(res_part1,1,nprim,real_wavetable,work ); 
	status= gsl_fft_real_transform(res_part2,1,nprim,real_wavetable,work ); 
	gsl_fft_real_wavetable_free(real_wavetable);
	gsl_fft_real_workspace_free(work);
       
	/*----------------------------------------*/
	/*             Normalisation              */
	/*----------------------------------------*/
	/* f(i) = (1/Nobs*Tech)*i                 */
	/* A(i) = (2/Nobs)*Tech*sqrt(Re*Re+Im*Im) */
	/* Nobs = Nb of data in the serie         */
	/* Tech = sampling period                 */
	/* Re, Im = real, imaginary part of FFT   */
	/*----------------------------------------*/
	fe=1.0/Trepet; 
	for (i=1;i<nprim;i+=2)
	  {
	    frequency[i]=fe/(2.0*nprim)*i;
	    ampli[i]=gsl_pow_2((res_part1[i]+res_part2[i])/2.0)+gsl_pow_2((res_part1[i+1]+res_part2[i+1])/2.0);
	    ampli[i]=(2.0*NINT(Trepet*24)/nprim)*sqrt(ampli[i]); /* Trepet expressed in hour*/
	  }

	free(res_part1);
	free(res_part2);

	/*---------------------------------------------------------------------------*/
	/* Averaging for freq-df/2 < freq <freq+df/2                                 */
	/*---------------------------------------------------------------------------*/
	
	if( (a=(double *) calloc(nprim/2,sizeof(double)) ) == NULL ) quit("can't allocate a");
	if( (f=(double *) calloc(nprim/2,sizeof(double)) ) == NULL ) quit("can't allocate f");
	
	for(i=1,j=0;i<nprim;i+=2,j++) a[j]=ampli[i]; 
	for(i=1,j=0;i<nprim;i+=2,j++) f[j]=frequency[i];
	free(ampli);
	free(frequency);
	
	trecord=0.5*(time[nmes-1]-time[0])/(24*365.25); /*record period of the half of initial serie, in year */
	df=1+floor(trecord)/0.5;               /* df frequencies ( equiv. 0.5 years) average (!!!????!!) */
	
	if( (ai=(double *)calloc(df,sizeof(double)) ) == NULL ) quit("can't allocate ai");
	if( (mean_fi=(double *)calloc((nprim),sizeof(double)) ) == NULL ) quit("can't allocate mean_fi");
	if( (rms_fi=(double *)calloc((nprim),sizeof(double)) ) == NULL ) quit("can't allocate rms_fi");
	
	for(i=0;i<(nprim/2-df);i++)
	  {
	    for(j=i;j<i+df;j++) ai[j-i]=a[j];
	    mean_fi[i]=gsl_stats_mean(ai,1,df);
	    rms_fi[i]=gsl_stats_sd(ai,1,df);
	  }
	free(ai);
  
	sprintf(outfilename,"%s/%s.smoothed_fft",rpath,stationname);
	out=fopen(outfilename,"w");
	fprintf(out,"# --freq ---- ampl - mean over df - rms over df\n");
	for(i=0;i<=(nprim/2-df);i++) /*error: (n-df)/2 before */
	  fprintf(out,"%9.5f %9.5f %9.5f %9.5f\n",f[i+df/2],a[i+df/2],mean_fi[i],rms_fi[i]);
	
	/*-----------------------------------------------------------------------------------*/
	/* Noise level for the analysis of the wave k = mean PSD in the vincinity of f(wave) */
	/*-----------------------------------------------------------------------------------*/
	
	for(k=0;k<work_s.n;k++) 
	  for(i=0;i<(nprim/2-df);i++)
	    if( fabs(work_s.wave[k].omega*24.0/360.0-f[i+df/2]) < (fe/nprim) ) /* freq in 1/day */
	      {
		/* linear interpolation between the frequency before and after f(wave)*/
		noiselevel[k]=(mean_fi[i+1]-mean_fi[i])/(fe/nprim)*(work_s.wave[k].omega*24.0/360.0-f[i+df/2])+mean_fi[i];
		break;
	      }
	
	free(f);
	free(a);
	free(mean_fi);
	free(rms_fi);
	break;

      }/* end case 'long serie' */

    }/*end switch*/

  /*---------------------------------*/
  /* FFT confidence interval limits  */
  /*---------------------------------*/

  chi_square_limits(df,&chi_inf,&chi_sup);

  sprintf(outfilename,"%s/chi2.gnu",rpath);
  out=fopen(outfilename,"w");
  fprintf(out," %9.5f %9.5f\n",(df/chi_inf),(df/chi_sup));
  fclose(out);



}




